// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Java Audio Client Implementation                                 ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include <stdio.h>
#include <jni.h>


#include "AudioClient.h"


#include "system.h"
#include "audioclient.h"
#include "audiodevice.h"
#include "audionull.h"
#include "audiodebug.h"
#include "transportinfo.h"
#include "trafficclassvalues.h"


using namespace Coral;


extern "C" {


// ###### Get the AudioClient instance from java (32 or 64 bit version) #####
#define GET_INSTANCE32 \
const jclass classID   = env->GetObjectClass(thisObject); \
const jfieldID fieldID = env->GetFieldID(classID,"AudioClientInstance32","I"); \
AudioClient* client    = (AudioClient*)env->GetIntField(thisObject,fieldID); \

#define GET_INSTANCE64 \
const jclass classID   = env->GetObjectClass(thisObject); \
const jfieldID fieldID = env->GetFieldID(classID,"AudioClientInstance64","J"); \
AudioClient* client    = (AudioClient*)env->GetLongField(thisObject,fieldID); \

#if (CPU_BITS == 32)
#define GET_INSTANCE GET_INSTANCE32
#elif (CPU_BITS == 64)
#define GET_INSTANCE GET_INSTANCE64
#else
#error "Unknown CPU_BITS definition!"
#endif


// ######## C++ constructor #################################################
JNIEXPORT jboolean JNICALL Java_AudioClient_constructIt(JNIEnv* env,
                                                        jobject thisObject,
                                                        jint    outputDevice)
{
   const jclass classID = env->GetObjectClass(thisObject);
#if (CPU_BITS == 32)
   const jfieldID clientFieldID = env->GetFieldID(classID,"AudioClientInstance32","I");
   const jfieldID deviceFieldID = env->GetFieldID(classID,"AudioDeviceInstance32","I");
#elif (CPU_BITS == 64)
   const jfieldID clientFieldID = env->GetFieldID(classID,"AudioClientInstance64","J");
   const jfieldID deviceFieldID = env->GetFieldID(classID,"AudioDeviceInstance64","J");
#else
#error "Unknown CPU_BITS definition!"
#endif

   AudioWriterInterface* device;
   if((outputDevice >= 0) && (outputDevice < 1000)) {
      device = new AudioDevice();
      if((device != NULL) && (!device->ready())) {
         delete device;
         cerr << "AudioDevice not ready -> Using AudioNull instead!" << endl;
         device = new AudioNull();
      }
   }
   else if(outputDevice >= 1000) {
      device = new AudioDebug();
   }
   else {
      device = new AudioNull();
   }

   AudioClient* client = NULL;
   if(device != NULL) {
      client = new AudioClient(device,NULL);
      if(client == NULL) {
         delete device;
         device = NULL;
      }
   }

#if (CPU_BITS == 32)
   env->SetIntField(thisObject,clientFieldID,(jint)client);
   env->SetIntField(thisObject,deviceFieldID,(jint)device);
#elif (CPU_BITS == 64)
   env->SetLongField(thisObject,clientFieldID,(jlong)client);
   env->SetLongField(thisObject,deviceFieldID,(jlong)device);
#else
#error "Unknown CPU_BITS definition!"
#endif
   return(client != NULL);
}


// ######## C++ destructor ##################################################
JNIEXPORT void JNICALL Java_AudioClient_destructIt(JNIEnv* env,
                                                   jobject thisObject)
{
   const jclass classID = env->GetObjectClass(thisObject);
#if (CPU_BITS == 32)
   const jfieldID clientFieldID = env->GetFieldID(classID,"AudioClientInstance32","I");
   const jfieldID deviceFieldID = env->GetFieldID(classID,"AudioDeviceInstance32","I");
   AudioClient* client          = (AudioClient*)env->GetIntField(thisObject,clientFieldID);
   AudioWriterInterface* device = (AudioWriterInterface*)env->GetIntField(thisObject,deviceFieldID);
#elif (CPU_BITS == 64)
   const jfieldID clientFieldID = env->GetFieldID(classID,"AudioClientInstance64","J");
   const jfieldID deviceFieldID = env->GetFieldID(classID,"AudioDeviceInstance64","J");
   AudioClient* client          = (AudioClient*)env->GetLongField(thisObject,clientFieldID);
   AudioWriterInterface* device = (AudioWriterInterface*)env->GetLongField(thisObject,deviceFieldID);
#else
#error "Unknown CPU_BITS definition!"
#endif

   if(client != NULL) {
      delete client;
   }
   if(device != NULL) {
      delete device;
   }

#if (CPU_BITS == 32)
   env->SetIntField(thisObject,clientFieldID,(jint)0);
   env->SetIntField(thisObject,deviceFieldID,(jint)0);
#elif (CPU_BITS == 64)
   env->SetLongField(thisObject,clientFieldID,(jlong)0);
   env->SetLongField(thisObject,deviceFieldID,(jlong)0);
#else
#error "Unknown CPU_BITS definition!"
#endif
}


// ###### Kill java interpreter #############################################
JNIEXPORT void JNICALL Java_AudioClient_killJavaInterpreter()
{
   kill(getpid(),SIGINT);

   // If java does not exit, kill it!
   kill(getpid(),SIGKILL);
}


// ######## play() implementation ###########################################
JNIEXPORT jboolean JNICALL Java_AudioClient_play(JNIEnv* env,
                                                 jobject thisObject,
                                                 jstring serverString,
                                                 jstring mediaNameString)
{
   const char* server     = env->GetStringUTFChars(serverString,0);
   const char* mediaName  = env->GetStringUTFChars(mediaNameString,0);
   GET_INSTANCE
   return(client->play(server,mediaName));
}


// ######## change() implementation ##########################################
JNIEXPORT void JNICALL Java_AudioClient_change(JNIEnv* env,
                                               jobject thisObject,
                                               jstring newMediaName)
{
   const char* mediaName  = env->GetStringUTFChars(newMediaName,0);
   GET_INSTANCE
   client->change(mediaName);
}


// ######## stop() implementation ###########################################
JNIEXPORT void JNICALL Java_AudioClient_stop(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   client->stop();
}


// ######## getMaxPosition() implementation ##################################
JNIEXPORT jlong JNICALL Java_AudioClient_getMaxPosition(
                           JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(client->getMaxPosition());
}


// ######## getPosition() implementation ##################################
JNIEXPORT jlong JNICALL Java_AudioClient_getPosition(
                           JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(client->getPosition());
}


// ######## getSamplingRate() implementation ################################
JNIEXPORT jint JNICALL Java_AudioClient_getSamplingRate(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(client->getSamplingRate());
}


// ######## getChannels() implementation ####################################
JNIEXPORT jint JNICALL Java_AudioClient_getChannels(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE 
   return(client->getChannels());
}


// ######## getBits() implementation ########################################
JNIEXPORT jint JNICALL Java_AudioClient_getBits(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE 
   return(client->getBits());
}


// ######## getByteOrder() implementation ###################################
JNIEXPORT jint JNICALL Java_AudioClient_getByteOrder(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE 
   return(client->getByteOrder());
}


// ######## getBitsPerSample() implementation ###############################
JNIEXPORT jint JNICALL Java_AudioClient_getBitsPerSample(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE 
   return(client->getBitsPerSample());
}


// ######## getBytesPerSecond() implementation ##############################
JNIEXPORT jint JNICALL Java_AudioClient_getBytesPerSecond(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE 
   return(client->getBytesPerSecond());
}


// ######## getRawBytesPerSecond() implementation ###########################
JNIEXPORT jint JNICALL Java_AudioClient_getRawBytesPerSecond(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE 
   return(client->getRawBytesPerSecond());
}

// ######## getBytesReceived() implementation ###############################
JNIEXPORT jlong JNICALL Java_AudioClient_getBytesReceived(
                           JNIEnv* env, jobject thisObject,
                           jint layer)
{
   GET_INSTANCE 
   return(client->getBytesReceived(layer));
}


// ######## getPacketsReceived() implementation #############################
JNIEXPORT jlong JNICALL Java_AudioClient_getPacketsReceived(
                           JNIEnv* env, jobject thisObject,
                           jint layer)
{
   GET_INSTANCE 
   return(client->getPacketsReceived(layer));
}


// ###### Get TransportInfoLevel ############################################
JNIEXPORT void JNICALL Java_AudioClient_getMediaInfo(
                          JNIEnv* env, jobject thisObject,
                          const jobject objectMI)
{
   GET_INSTANCE
   MediaInfo mediaInfo = client->getMediaInfo();
   const jclass classMI = env->GetObjectClass(objectMI);

   env->SetLongField(objectMI,env->GetFieldID(classMI,"StartTimeStamp","J"),
                     mediaInfo.StartTimeStamp);
   env->SetLongField(objectMI,env->GetFieldID(classMI,"EndTimeStamp","J"),
                     mediaInfo.EndTimeStamp);
   env->SetObjectField(objectMI,env->GetFieldID(classMI,"Title","Ljava/lang/String;"),
                       env->NewStringUTF(mediaInfo.Title));
   env->SetObjectField(objectMI,env->GetFieldID(classMI,"Artist","Ljava/lang/String;"),
                       env->NewStringUTF(mediaInfo.Artist));
   env->SetObjectField(objectMI,env->GetFieldID(classMI,"Comment","Ljava/lang/String;"),
                       env->NewStringUTF(mediaInfo.Comment));
}


// ###### Get TransportInfoLevel ############################################
void getTransportInfoLevel(JNIEnv* env, jobject objectLevel, const jclass classLevel,
                           const TransportInfoLevel& level)
{
   // ====== Copy level =====================================================
   env->SetIntField(objectLevel,env->GetFieldID(classLevel,"BytesPerSecondScale","I"),
                       level.BytesPerSecondScale);
   env->SetIntField(objectLevel,env->GetFieldID(classLevel,"PacketsPerSecondScale","I"),
                       level.PacketsPerSecondScale);
   env->SetIntField(objectLevel,env->GetFieldID(classLevel,"FramesPerSecond","I"),
                       level.FramesPerSecond);
   env->SetIntField(objectLevel,env->GetFieldID(classLevel,"FramesPerSecondScale","I"),
                       level.FramesPerSecondScale);
   env->SetIntField(objectLevel,env->GetFieldID(classLevel,"MaxTransferDelay","I"),
                       level.MaxTransferDelay);
   env->SetIntField(objectLevel,env->GetFieldID(classLevel,"Quality","I"),
                       level.Quality);
   env->SetIntField(objectLevel,env->GetFieldID(classLevel,"LevelUp","I"),
                       level.LevelUp);
   env->SetIntField(objectLevel,env->GetFieldID(classLevel,"LevelDown","I"),
                       level.LevelDown);
   env->SetIntField(objectLevel,env->GetFieldID(classLevel,"QualityLayers","I"),
                       level.QualityLayers);

   // ====== Copy layers ====================================================
   const jfieldID idLayer  = env->GetFieldID(classLevel,"QualityLayer","[LTransportInfoLayer;");
   jobjectArray arrayLayer = (jobjectArray)env->GetObjectField(objectLevel,idLayer);
   for(cardinal i = 0;i < min((cardinal)level.QualityLayers,TransportInfoLevel::MaxQualityLayers);i++) {
      jobject objectLayer     = (jlongArray)env->GetObjectArrayElement(arrayLayer,(jsize)i);
      const jclass classLayer = env->GetObjectClass(objectLayer);

      env->SetLongField(objectLayer,env->GetFieldID(classLayer,"BytesPerSecond","J"),
                          level.QualityLayer[i].BytesPerSecond);
      env->SetIntField(objectLayer,env->GetFieldID(classLayer,"PacketsPerSecond","I"),
                          level.QualityLayer[i].PacketsPerSecond);
      env->SetIntField(objectLayer,env->GetFieldID(classLayer,"FrameSize","I"),
                          level.QualityLayer[i].FrameSize);
      env->SetIntField(objectLayer,env->GetFieldID(classLayer,"MaxLossRate","I"),
                          level.QualityLayer[i].MaxLossRate);
      env->SetIntField(objectLayer,env->GetFieldID(classLayer,"Flags","I"),
                          level.QualityLayer[i].Flags);
   }
}


// ###### getTransportInfo() implementation #################################
JNIEXPORT void JNICALL Java_AudioClient_getTransportInfo(
                          JNIEnv* env, jobject thisObject, jobject objectTI)
{
   GET_INSTANCE 
   TransportInfo transportInfo = client->getTransportInfo();
   const jclass classTI = env->GetObjectClass(objectTI);


   // ====== Copy wanted values =============================================
   const jfieldID idMinBytesPerSecond    = env->GetFieldID(classTI,"MinBytesPerSecond","[J");
   const jfieldID idMaxBytesPerSecond    = env->GetFieldID(classTI,"MaxBytesPerSecond","[J");
   const jfieldID idWantedBytesPerSecond = env->GetFieldID(classTI,"WantedBytesPerSecond","[J");
   jlongArray arrayMinBytesPerSecond     = (jlongArray)env->GetObjectField(objectTI,idMinBytesPerSecond);
   jlongArray arrayMaxBytesPerSecond     = (jlongArray)env->GetObjectField(objectTI,idMaxBytesPerSecond);
   jlongArray arrayWantedBytesPerSecond  = (jlongArray)env->GetObjectField(objectTI,idWantedBytesPerSecond);

   const jfieldID idMinPacketsPerSecond    = env->GetFieldID(classTI,"MinPacketsPerSecond","[I");
   const jfieldID idMaxPacketsPerSecond    = env->GetFieldID(classTI,"MaxPacketsPerSecond","[I");
   const jfieldID idWantedPacketsPerSecond = env->GetFieldID(classTI,"WantedPacketsPerSecond","[I");
   jintArray arrayMinPacketsPerSecond      = (jintArray)env->GetObjectField(objectTI,idMinPacketsPerSecond);
   jintArray arrayMaxPacketsPerSecond      = (jintArray)env->GetObjectField(objectTI,idMaxPacketsPerSecond);
   jintArray arrayWantedPacketsPerSecond   = (jintArray)env->GetObjectField(objectTI,idWantedPacketsPerSecond);

   const jfieldID idWantedMaxLossRate = env->GetFieldID(classTI,"WantedMaxLossRate","[I");
   jintArray arrayWantedMaxLossRate   = (jintArray)env->GetObjectField(objectTI,idWantedMaxLossRate);

   jlong minBytesPerSecondArray[TransportInfoLevel::MaxQualityLayers];
   jlong maxBytesPerSecondArray[TransportInfoLevel::MaxQualityLayers];
   jlong wantedBytesPerSecondArray[TransportInfoLevel::MaxQualityLayers];
   jint  minPacketsPerSecondArray[TransportInfoLevel::MaxQualityLayers];
   jint  maxPacketsPerSecondArray[TransportInfoLevel::MaxQualityLayers];
   jint  wantedPacketsPerSecondArray[TransportInfoLevel::MaxQualityLayers];
   jint  wantedMaxLossRate[TransportInfoLevel::MaxQualityLayers];
   for(cardinal i = 0;i < TransportInfoLevel::MaxQualityLayers;i++) {
      minBytesPerSecondArray[i]      = transportInfo.WantedBytesPerSecond[i].getMin();
      maxBytesPerSecondArray[i]      = transportInfo.WantedBytesPerSecond[i].getMax();
      wantedBytesPerSecondArray[i]   = transportInfo.WantedBytesPerSecond[i].getValue();
      minPacketsPerSecondArray[i]    = transportInfo.WantedPacketsPerSecond[i].getMin();
      maxPacketsPerSecondArray[i]    = transportInfo.WantedPacketsPerSecond[i].getMax();
      wantedPacketsPerSecondArray[i] = transportInfo.WantedPacketsPerSecond[i].getValue();
      wantedMaxLossRate[i]           = transportInfo.WantedMaxLossRate[i];
   }

   env->SetLongArrayRegion(arrayMinBytesPerSecond,(jsize)0,(jsize)TransportInfoLevel::MaxQualityLayers,(jlong*)minBytesPerSecondArray);
   env->SetLongArrayRegion(arrayMaxBytesPerSecond,(jsize)0,(jsize)TransportInfoLevel::MaxQualityLayers,(jlong*)maxBytesPerSecondArray);
   env->SetLongArrayRegion(arrayWantedBytesPerSecond,(jsize)0,(jsize)TransportInfoLevel::MaxQualityLayers,(jlong*)wantedBytesPerSecondArray);

   env->SetIntArrayRegion(arrayMinPacketsPerSecond,(jsize)0,(jsize)TransportInfoLevel::MaxQualityLayers,(jint*)minPacketsPerSecondArray);
   env->SetIntArrayRegion(arrayMaxPacketsPerSecond,(jsize)0,(jsize)TransportInfoLevel::MaxQualityLayers,(jint*)maxPacketsPerSecondArray);
   env->SetIntArrayRegion(arrayWantedPacketsPerSecond,(jsize)0,(jsize)TransportInfoLevel::MaxQualityLayers,(jint*)wantedPacketsPerSecondArray);

   env->SetIntArrayRegion(arrayWantedMaxLossRate,(jsize)0,(jsize)TransportInfoLevel::MaxQualityLayers,(jint*)wantedMaxLossRate);

   
   env->SetIntField(objectTI,env->GetFieldID(classTI,"MinFramesPerSecond","I"),
                       transportInfo.WantedFramesPerSecond.getMin());
   env->SetIntField(objectTI,env->GetFieldID(classTI,"MaxFramesPerSecond","I"),
                       transportInfo.WantedFramesPerSecond.getMax());
   env->SetIntField(objectTI,env->GetFieldID(classTI,"WantedFramesPerSecond","I"),
                       transportInfo.WantedFramesPerSecond.getValue());
   env->SetIntField(objectTI,env->GetFieldID(classTI,"WantedMaxTransferDelay","I"),
                       transportInfo.WantedMaxTransferDelay);
   env->SetIntField(objectTI,env->GetFieldID(classTI,"StartFramesPerSecond","I"),
                       transportInfo.StartFramesPerSecond);
   env->SetIntField(objectTI,env->GetFieldID(classTI,"QualityLevels","I"),
                       transportInfo.QualityLevels);
   env->SetIntField(objectTI,env->GetFieldID(classTI,"QualityLayers","I"),
                       transportInfo.QualityLayers);
   env->SetIntField(objectTI,env->GetFieldID(classTI,"Flags","I"),
                       transportInfo.Flags);


   // ====== Copy levels ====================================================
   const jfieldID idLevel  = env->GetFieldID(classTI,"QualityLevel","[LTransportInfoLevel;");
   jobjectArray arrayLevel = (jobjectArray)env->GetObjectField(objectTI,idLevel);
   for(cardinal i = 0;i < min((cardinal)transportInfo.QualityLevels,TransportInfo::MaxQualityLevels);i++) {
      jobject objectLevel     = (jlongArray)env->GetObjectArrayElement(arrayLevel,(jsize)i);
      const jclass classLevel = env->GetObjectClass(objectLevel);
      getTransportInfoLevel(env,objectLevel,classLevel,transportInfo.QualityLevel[i]);
   }


   // ====== Copy current settings ==========================================
   const jfieldID idCurrentSetting  = env->GetFieldID(classTI,"CurrentSetting","LTransportInfoLevel;");
   jobject objectCurrentSetting     = (jobject)env->GetObjectField(objectTI,idCurrentSetting);
   const jclass classCurrentSetting = env->GetObjectClass(objectCurrentSetting);
   getTransportInfoLevel(env,objectCurrentSetting,classCurrentSetting,transportInfo.CurrentSetting);
}


// ######## getErrorCode() implementation ###################################
JNIEXPORT jint JNICALL Java_AudioClient_getErrorCode(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(client->getErrorCode());
}


// ######## getIPVersion() implementation ###################################
JNIEXPORT jint JNICALL Java_AudioClient_getIPVersion(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(client->getIPVersion());
}


// ######## getEncoding() implementation ####################################
JNIEXPORT jstring JNICALL Java_AudioClient_getEncoding(
                             JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(env->NewStringUTF(client->getEncoding()));
}


// ######## getBandwidthLimit() implementation ##############################
JNIEXPORT jint JNICALL Java_AudioClient_getBandwidthLimit(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(client->getBandwidthLimit());
}


// ######## getEncodingName() implementation ################################
JNIEXPORT jstring JNICALL Java_AudioClient_getEncodingName(
                             JNIEnv* env, jobject thisObject, jint index)
{
   GET_INSTANCE
   const char* name = client->getEncodingName(index);
   if(name == NULL) name = "";
   return(env->NewStringUTF(name));
}

// ######## getValidSamplingRate() implementation ###########################
JNIEXPORT jint JNICALL Java_AudioClient_getValidSamplingRate(
                             JNIEnv* env, jobject thisObject, jint index)
{
   if(index >= (int)AudioQuality::ValidRates)
      return(0);
   else if(index < 0)
      return(0);
   return(AudioQuality::ValidRatesTable[index]);
}

// ######## getValidBits() implementation ###################################
JNIEXPORT jint JNICALL Java_AudioClient_getValidBits(
                             JNIEnv* env, jobject thisObject, jint index)
{
   if(index >= (int)AudioQuality::ValidBits)
      return(0);
   else if(index < 0)
      return(0);
   return(AudioQuality::ValidBitsTable[index]);
}

// ######## getServerAddressString() implementation #########################
JNIEXPORT jstring JNICALL Java_AudioClient_getServerAddressString(
                             JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(env->NewStringUTF(client->getServerAddressString().getData()));
}


// ######## getFlowLabel() implementation ###################################
JNIEXPORT jint JNICALL Java_AudioClient_getFlowLabel(
                          JNIEnv* env, jobject thisObject, jint layer)
{
   GET_INSTANCE
   return(client->getFlowLabel(layer));
}


// ######## getTrafficClass() implementation ################################
JNIEXPORT jint JNICALL Java_AudioClient_getTrafficClass(
                          JNIEnv* env, jobject thisObject, jint layer)
{
   GET_INSTANCE
   return(client->getTrafficClass(layer));
}


// ######## getNameForTrafficClass() implementation #########################
JNIEXPORT jstring JNICALL Java_AudioClient_getNameForTrafficClass(
                             JNIEnv* env, jobject thisObject, jint trafficClass)
{
   char str[32];
   const char* tname = TrafficClassValues::getNameForTrafficClass(trafficClass);
   if(tname == NULL) {
      snprintf((char*)&str,sizeof(str),"$%02x",(int)trafficClass);
      tname = (const char*)&str;
   }
   return(env->NewStringUTF(tname));
}


// ######## getLayers() implementation ######################################
JNIEXPORT jint JNICALL Java_AudioClient_getLayers(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(client->getLayers());
}


// ######## getOurAddressString() implementation ############################
JNIEXPORT jstring JNICALL Java_AudioClient_getOurAddressString(
                             JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(env->NewStringUTF(client->getOurAddressString().getData()));
}


// ######## playing() implementation ########################################
JNIEXPORT jboolean JNICALL Java_AudioClient_playing(
                              JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(client->playing());
}


// ######## getServerSSRC() implementation ##################################
JNIEXPORT jint JNICALL Java_AudioClient_getServerSSRC(
                          JNIEnv* env, jobject thisObject,
                          jint layer)
{
   GET_INSTANCE
   return(client->getServerSSRC(layer));
}


// ######## getOurSSRC() implementation #####################################
JNIEXPORT jint JNICALL Java_AudioClient_getOurSSRC(
                          JNIEnv* env, jobject thisObject)
{
   GET_INSTANCE
   return(client->getOurSSRC());
}


// ######## getPacketsLost() implementation #################################
JNIEXPORT jlong JNICALL Java_AudioClient_getPacketsLost(
                           JNIEnv* env, jobject thisObject,
                           jint layer)
{
   GET_INSTANCE
   return(client->getPacketsLost(layer));
}


// ######## getFractionLost() implementation ################################
JNIEXPORT jdouble JNICALL Java_AudioClient_getFractionLost(
                             JNIEnv* env, jobject thisObject,
                             jint layer)
{
   GET_INSTANCE
   return(client->getFractionLost(layer));
}


// ######## getJitter() implementation ################################
JNIEXPORT jdouble JNICALL Java_AudioClient_getJitter(
                             JNIEnv* env, jobject thisObject,
                             jint layer)
{
   GET_INSTANCE
   return(client->getJitter(layer));
}


// ######## setPosition() implementation ####################################
JNIEXPORT void JNICALL Java_AudioClient_setPosition(
                          JNIEnv* env, jobject thisObject, jlong value)
{
   GET_INSTANCE
   client->setPosition(value);
}


// ######## setPause() implementation #######################################
JNIEXPORT void JNICALL Java_AudioClient_setPause(
                          JNIEnv* env, jobject thisObject, jboolean on)
{
   GET_INSTANCE
   client->setPause(on);
}


// ######## setSamplingRate() implementation ################################
JNIEXPORT void JNICALL Java_AudioClient_setSamplingRate(
                          JNIEnv* env, jobject thisObject, jint value)
{
   GET_INSTANCE
   client->setSamplingRate(value);
}


// ######## setChannels() implementation ####################################
JNIEXPORT void JNICALL Java_AudioClient_setChannels(
                          JNIEnv* env, jobject thisObject, jint value)
{
   GET_INSTANCE
   client->setChannels(value);
}


// ######## setBits() implementation ########################################
JNIEXPORT void JNICALL Java_AudioClient_setBits(
                          JNIEnv* env, jobject thisObject, jint value)
{
   GET_INSTANCE
   client->setBits(value);
}


// ######## setEncoding() implementation ####################################
JNIEXPORT void JNICALL Java_AudioClient_setEncoding(
                          JNIEnv* env, jobject thisObject, jint value)
{
   GET_INSTANCE
   client->setEncoding(value);
}


// ######## setBandwidthLimit() implementation ##############################
JNIEXPORT void JNICALL Java_AudioClient_setBandwidthLimit(
                          JNIEnv* env, jobject thisObject, jint bandwidthLimit)
{
   GET_INSTANCE
   client->setBandwidthLimit(bandwidthLimit);
}


}
