// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### TransportInfo Level                                              ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


import TransportInfoLayer;


/**
  * This class keeps information on a quality level.
  *
  * @short   TransportInfo Level
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
public class TransportInfoLevel {
   // ====== Constructor ====================================================
   /**
     * Constructor.
     */
   public TransportInfoLevel() {
      QualityLayer = new TransportInfoLayer[TransportInfoLevel.MaxQualityLayers];
      for(int i = 0;i < MaxQualityLayers;i++) {
         QualityLayer[i] = new TransportInfoLayer();
      }
   }


   // ====== Reset ==========================================================
   /**
     * Reset.
     */
   public void reset() {
      BytesPerSecondScale  = 0;
      FramesPerSecondScale = 0;
      FramesPerSecond      = 0;
      MaxTransferDelay     = 0;
      Quality              = 0;
      LevelUp              = 255;
      LevelDown            = 255;
      QualityLayers        = 0;
      for(int i = 0;i < MaxQualityLayers;i++) {
         QualityLayer[i].reset();
      }
   }


   // ====== Print ==========================================================
   /**
     * Print.
     */
   public void print() {
      System.out.println("   BytesPerSecondScale   = " + BytesPerSecondScale);
      System.out.println("   PacketsPerSecondScale = " + PacketsPerSecondScale);
      System.out.println("   FramesPerSecondScale  = " + FramesPerSecondScale);
      System.out.println("   FramesPerSecond       = " + FramesPerSecond);
      System.out.println("   MaxTransferDelay      = " + (MaxTransferDelay / 16) + " [ms]");
      System.out.println("   Quality               = " + Quality);
      System.out.println("   LevelUp               = " + LevelUp);
      System.out.println("   LevelDown             = " + LevelDown);
      System.out.println("   QualityLayers         = " + QualityLayers);
      for(int i = 0;i < QualityLayers;i++) {
         System.out.println("   Layer #" + i + ":");
         QualityLayer[i].print();
      }
   }


   // ====== Constants ======================================================
   /**
     * Constant for maximum number of quality layers.
     */
   public final static int MaxQualityLayers = 4;


   // ====== Level's values =================================================
   /**
     * Level's bytes per second scale factor multiplied with 65536.
     */
   public int BytesPerSecondScale;

   /**
     * Level's packets per second scale factor multiplied with 65536.
     */
   public int PacketsPerSecondScale;

   /**
     * Level's frames per second scale factor multiplied with 65536.
     */
   public int FramesPerSecondScale;

   /**
     * Level's frames per second: StartFramesPerSecond * FramesPerSecondScale.
     */
   public int FramesPerSecond;

   /**
     * Level's maximum transfer delay for base data in 1/16 milliseonds.
     */
   public int MaxTransferDelay;

   /**
     * Level's quality in fraction of 255.
     */
   public int Quality;

   /**
     * Next higher quality level's number.
     */
   public int LevelUp;

   /**
     * Next lower quality level's number.
     */
   public int LevelDown;

   /**
     * Number of layers in the following array.
     */
   public int QualityLayers;


   // ====== Level's layers =================================================
   /**
     * Array of layer descriptions.
     */
   public TransportInfoLayer QualityLayer[];
};
