#include "qosmanager.h"
#include <iostream>
#include <algorithm>

namespace Coral {

QoSManager::QoSManager()
  : TimedThread(1000000)
{
  synchronized();                   // thread synchronisation
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : QoSManager" << endl;
  #endif


  initRTTPinger();


  scaleOut = new ofstream("/tmp/scales.dat");
  if(!scaleOut->good()){ 
     cerr << "FEHLER #0" << endl;
  }

  qualityOut = new ofstream("/tmp/quality.dat");
  if(!qualityOut->good()){ 
     cerr << "FEHLER #0.5" << endl;
  }


  outfile = new ofstream("/tmp/rtt.log");
  gnuscript = new ofstream("/tmp/rtt.gp");

  daten = new ofstream("/tmp/rtt.dat");


  if(!daten->good()) {
     cerr << "FEHLER #1" << endl;
  }
  if(!gnuscript->good()) {
     cerr << "FEHLER #2" << endl;
  }

  time_t timeStamp = (getMicroTime() / 1000000);
  const struct tm* timeptr = localtime(&timeStamp);
  char str[256];
  strftime((char*)&str,256,"%A, %B %d %G %T %Z",timeptr);
  String infoString = "Generated on " + (String)str;

  *gnuscript << "set terminal postscript enhanced" << endl;
  *gnuscript << "set encoding iso_8859_1" << endl;
  *gnuscript << "set title \"Round Trip Time\" \"Times-Roman,16\"" << endl;
  *gnuscript << "set timestamp \"" << infoString.getData() << "\" 0,0 \"Times-Roman,12\"" << endl;
  *gnuscript << "set ylabel \"Round Trip Time [ms]\"" << endl;
  *gnuscript << "set xlabel \"Time [s]\"" << endl;


  calculationInterval = 10;         // set interval for new mappings       

  ClassBandwidthMapType cbmt;       // from reservation-modul

  // get information from the reservation-modul
  ResvMod.Get_BandwidthInfo(&cbmt);

  for(int i=0; i < NUM_PRIOS; i++) {
    prio[i]=cbmt.prio[i] / 8;
    freeInPrio[i] = prio[i];

    //    cout << "Priority Class " << i << " :  "<< prio[i] << endl;
  }
  totalBandwidth = cbmt.wholeBandwidth / 8;

  unsynchronized();
  start();                          // start the interval-timer.
}


QoSManager::~QoSManager()
{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : ~QoSManager" << endl;
  #endif
  rttp->deactivateLogger();
  rttp->stop();
}

void QoSManager::initRTTPinger()
{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : initRTTPinger" << endl;
  #endif
   // ====== Check EUID =====================================================
   uid_t uid = geteuid();
   if(uid != 0) {
      cerr << "ERROR: Only root is allowed to run this program!" << endl;
      exit(1);
   }

   // ====== Open sockets ===================================================
   Socket* pingSocket4 = new Socket(Socket::IPv4,Socket::Raw,Socket::ICMPv4);
   if(pingSocket4 == NULL) {
      cerr << "ERROR: Out of memory!" << endl;
      exit(1);
   }
   if(!pingSocket4->bind()) {
      cerr << "ERROR: Unable to bind socket for ICMPv4!" << endl;
      exit(1);
   }
   Socket* pingSocket6 = NULL;
   if(InternetAddress::hasIPv6()) {
      pingSocket6 = new Socket(Socket::IPv6,Socket::Raw,Socket::ICMPv6);
      if(pingSocket6 == NULL) {
         cerr << "ERROR: Out of memory!" << endl;
         exit(1);
      }
      if(!pingSocket6->bind()) {
         cerr << "ERROR: Unable to bind socket for ICMPv6!" << endl;
         exit(1);
      }
   }

   // Go back to real UID. Root access is not necessary below this point.
/* ????????
   uid = getuid();
   setuid(uid);
*/

   // ====== Create RoundTripTimePinger =====================================
   rttp = new RoundTripTimePinger(pingSocket4,pingSocket6);
   if(!rttp->ready()) {
      cerr << "ERROR: RoundTripTimePinger not ready!" << endl;
      exit(1);
   }

   /*
   // ====== Print status ===================================================
   cout << "Round Trip Time Pinger - Copyright (C) 1999/2000 Thomas Dreibholz" << endl;
   cout << "-----------------------------------------------------------------" << endl;
   cout << endl;
   for(cardinal i = 1;i < (cardinal)argc;i++) {
      getNewHost(*rttp,argv[i]);
   }
   if(pinger.getHosts() < 1) {
      cerr << "There are no hosts in list -> exit." << endl;
      exit(1);
   }
   */

   rttp->start();
}

card32 QoSManager::newSession(PortableAddress dest) {
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : newSession" << endl;
  #endif
  synchronized();
#ifdef PRINT_INFOS
  cout << "newSession" << endl;
#endif
  static card32 i = 0;
  i++;

  InternetAddress addr(dest);

  //add one pinger host for every class of this session
  int j=0;

  if(!rttp->ready()) {
    cerr << "RTT-Pinger not ready !" << endl;
  }

  card8 tos=0;
  for(j=0; j < NUM_PRIOS; j++) {
    tos = ResvMod.Get_Priority(j);
    if(prio[j] != 0) {
      rttp->addHost(addr,tos);
    }
  }

  //cout << (int) session.size() << endl;
  
  Session* s=new Session(this, dest);

  // cout << "qosmanager :" <<s.getQoSManager() << endl;

  s->setID(i);
  session.push_back(s);

  if(session.size()==1) {
#ifdef PRINT_INFOS
    cout << "activate rtt-logger" << endl;
#endif
    rttp->activateLogger(gnuscript,daten,"/tmp/rtt.dat");

// TD:
// Der Thread wurde schon bei initRTT gestartet!
//    rttp->start();
  }
  
  unsynchronized();
  return i;
}


bool QoSManager::closeSession(card32 id) {
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : closeSession" << endl;
  #endif
  synchronized();
  int i;
  vector<Session*>::iterator iter = session.begin();
  
  //cout << "closeSession" << endl;
  

  for(i=0; i < (int) session.size(); i++) {     // search the session
    if(session[i]->getID() == id) {             // session found?
      if( session[i]->getNumStreams() == 0 ) {  // session empty?
	InternetAddress addr(session[i]->getDestination());
	/*
	// remove all pinger hosts
	for(j=0; j < NUM_RIOS; j++) {
	  tos = ResvMod.Get_Priority(j);
	  if(prio[j] != 0) {
	    rttp->removeHost(addr,tos);
	  }	  
	}
	*/
	delete(session[i]);
	session.erase(iter);                    // close session
	unsynchronized();
	return true;
      } else {
	unsynchronized();
	return false;                           // session not empty
      }
      iter++;
    }
  }
  unsynchronized();
  return false;
}

bool QoSManager::setSessionWeight(card32 sessionId, card32 weight) {
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : setSessionWeight" << endl;
  #endif
  synchronized();
  int i;
  for(i=0; i < (int) session.size(); i++) {
    if( sessionId == session[i]->getID() ) {
      session[i]->setWeight(weight);
      unsynchronized();
      return true;
    }
  }
  unsynchronized();
  return false;
}

card32 QoSManager::getSessionWeight(card32 sessionID) {
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : getSessionWeight" << endl;
  #endif
  synchronized();
  int i;
  for(i=0; i < (int) session.size(); i++) {
    if(sessionID == session[i]->getID()) {
      unsynchronized();
      return session[i]->getWeight();
    }
  }
  unsynchronized();
  return 0;

}

int QoSManager::getID()
{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : getID" << endl;
  #endif
  // cout << "getID" << endl;
  synchronized();
  static card32 i = 0;
  i++;
  unsynchronized();
  return i;
}

// returns whether a1 and a2 have the same inet-address.
// the portnumber is ignored !!!
bool compareAddresses(PortableAddress a1, PortableAddress a2)
{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : compareAddresses" << endl;
  #endif
   return((a1.Host[0] == a2.Host[0]) &&
          (a1.Host[1] == a2.Host[1]) &&
          (a1.Host[2] == a2.Host[2]) &&
          (a1.Host[3] == a2.Host[3]) &&
          (a1.Host[4] == a2.Host[4]) &&
          (a1.Host[5] == a2.Host[5]) &&
 	  (a1.Host[6] == a2.Host[6]) &&
	  (a1.Host[7] == a2.Host[7]));
}

card32 QoSManager::newStream(card32 streamID, ExtendedTransportInfo ti, QoSManagerInterface* interface)
{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : newStream" << endl;
  #endif
  //cout << "newStream" << endl;
  synchronized();
  int i=0,j=0;        // counter

  scaleCount = 0;

  card32 sessionIndex =0;           // index of the session in session-vector for the new stream.
  bool   sessionFound = false;      // when no session will be found with the same destination ,
                                    // a new session must be created.


  //  cout << "newStream #Levels " << (int)ti.QualityLevels << endl;
  //  cout << "newStream startfps: " << ti.StartFramesPerSecond << endl;
  //  cout << "newStream level 21 fpsscale: " << ti.QualityLevel[21].FramesPerSecondScale / 65536. << endl;
  //  cout << "newStream layer0 framesize: " << ti.QualityLevel[21].QualityLayer[0].FrameSize << endl;


  // ===== search a sesison for this stream
  for(i=0; i < (int) session.size(); i++) {
    if(compareAddresses(session[i]->getDestination(), ti.SrcDest[0].Destination)) {
      InternetAddress s(ti.SrcDest[0].Destination);

#ifdef PRINT_INFOS
      cout << "added stream to Session : " << s << endl;
#endif

      sessionIndex = (card32) i;
      sessionFound = true;
      break;
    }
  }

  // ===== create a new session ? =======
  if(!sessionFound) {
    InternetAddress s(ti.SrcDest[0].Destination);
#ifdef PRINT_INFOS
    cout << "new Session to : " << s << endl;
#endif
    newSession(ti.SrcDest[0].Destination);
    sessionIndex = session.size()-1;
  }  

  // cout << "sessionIndex: " <<  sessionIndex << endl;
  // cout << "sessionID: " << session[sessionIndex]->getID() << endl;
  // cout << "sessionAddress: " << session[sessionIndex] << endl;

  // ===== create the new stream ==========================================================

  

  LayeredStream* newStreamP=new LayeredStream(streamID, ti, interface, session[sessionIndex]);

  // ===== add the new stream to its session ==============================================
  session[sessionIndex]->addStream(newStreamP);

  //  LayeredStream* newStreamP = session[sessionIndex]->getStream( session[sessionIndex]->getNumStreams()-1 ); 

  //newStreamP -> correctPointer();

  // ===== calculate level with witch the new stream should come ==========================
  int level;
  card32 quality;
  card32 qualitySum=0;
  card32 numStreams=0;

  for(i=0; i < (int)session.size(); i++) {                // for each session
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {     // for each flow in this session
      level = session[i]->getStream(j)->getLevel();
      if(level != 255) {
	quality = session[i]->getStream(j)->getLevel(level)->getQuality();
	qualitySum += quality;
      }
      numStreams++;
    }
  }
  //cout << "2" << endl;
  
  if(qualitySum == 0) {
    quality=255; // <-------------------------------- qualitaet des 1.stroms ???
  } else {
    quality = qualitySum / (numStreams);
  }

  level = -1;
  for(i=0; i < ti.QualityLevels; i++) {
    level ++;
    if(ti.QualityLevel[i].Quality >= quality) {
      break;
    }
  }


  //cout << "#stroeme : " << numStreams << endl;
  //cout << "qualitysum : " << qualitySum << endl;
  
  // =============================================================================================

  //cout << "versuche level " << (int) level << " mit Qualitaet " << quality  <<endl;


  // add the new stream to the priority classes
  addStreamToClasses(newStreamP, sessionIndex, level);

  //cout << "eingefuegt mit level " << (int) newStreamP-> getLevel() <<endl;

  //callback the application.
  /*
  (*scaleOut) << "new stream " << streamID << " : scaleCount=" << scaleCount << endl;

  (*qualityOut) << "newStream " << streamID << ":" << endl;
  for(i=0; i < (int)session.size(); i++) {                // for each session
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {     // for each flow in this session
      (*qualityOut) << "Stream " << session[i]->getStream(j)->getID()<< " : "<< (int)session[i]->getStream(j)->getQuality();
      (*qualityOut) << "(Level " << (int)session[i]->getStream(j)->getLevel() << ")" << endl;
      
    }
  }  
  */
  applicationCallback();
  
  //cout << "newStream beendet" << endl;
  unsynchronized();
  return streamID;
}

bool QoSManager::closeStream(card32 ID) {
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : closeStream" << endl;
  #endif
  (*scaleOut) << "closeStream "<< ID << endl;
  synchronized();
  int i=0,j=0;
  bool result;

  //cout <<  "sessions : " << (int) session.size() << endl; 

  for(i=0; i < (int) session.size(); i++) {
    //cout << "0" << endl;
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {
      if(session[i]->getStream(j)->getID() == ID) {
	//cout << "1" << endl;
	result = session[i]->closeStream(ID); // <---------------hier manchmal absturz
	if (session[i]->getNumStreams() == 0) {
	  //cout << "3" << endl;
	  //closeSession(session[i]->getID());
	}
	//cout << "closeStream beendet" << endl;
	unsynchronized();
	//cout << "strom geschlossen" << endl;
	return result;
      }
    }
  }

  //cout << "closeStream beendet" << endl;  
  unsynchronized();
  //cout << "strom nicht geschlossen" << endl;
  return false;
}


void QoSManager::calculateNewMapping()
{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : calculateNewMapping" << endl;
  #endif
   
  int i=0,j=0;

  scaleCount = 0;

  card8 tos = 0;

  //cout << "1" << endl;

  //update all RTT's
  for(i=0; i < (int)session.size(); i++) {
    for(j=0; j < NUM_PRIOS; j++) {
      tos = ResvMod.Get_Priority(j);
      session[i]->setDelayOfClass(j, rttp->getRoundTripTime(session[i]->getDestination(), tos)/2 );
      //cout << "delay of class " << j << ":" << session[i]->getDelayOfClass(j) << endl;
    }
  }

  LayeredStream* streamP;            // help-pointer to a stream
  vector<sortStruct> streamSort;     // vector for sort streams by their quality
                                     // and remember session and stream index of the streams
  sortStruct newElem;                // one sortStruct element

  // ===== remove all flows from priority classes ===================================
  for(i=0; i < NUM_PRIOS; i++) {                           // for all priority classes
    freeInPrio[i] = prio[i];                              // reset free bandwidth
  }

  
  for(i=0; i < (int)session.size(); i++) {                // for each session
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {     // for each flow in this session
     
      streamP = session[i]->getStream(j);
      
      // increment maximal allowed level for each stream
      if(streamP->getMaximalLevel() < streamP->getNumLevels() - 1 ) {
	streamP->setMaximalLevel( streamP->getLevel(streamP->getMaximalLevel())->getLevelUp() );
      }

      //streamP -> current_bw = 0;                            // reset flow
      streamP -> setLevel(255);
      // add the flow to the sort-vector
      
      //newElem.value = 0;  // quality of level 255 = 0 !
      //newElem.flowIndex = j;
      //newElem.sessionIndex = i;
      //flowSort.push_back(newElem);
      
    }
  }


  // ===== bring level 0 of each flow in classes ====================================
  bool scaleUpOk;
  card32 sessionIndex;
  card32 streamIndex;
  int level, nextLevel;
  card8 quality;


  for(i=0; i < (int)session.size(); i++) {                // for each session
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {     // for each flow in this session
      //cout << "level of stream: " << (int) session[i]->getStream(j)->getLevel() << endl;

      //cout << "session of this stream: " << session[i]->getStream(j)->getSession() << endl;

      scaleUpOk = session[i]->getStream(j)->scaleUp();
      //cout << "geschafft" << endl;
      if(scaleUpOk) {
	scaleCount++;
	// add the flow to the sort-vector
	nextLevel = session[i]->getStream(j)->getLevel(0)->getLevelUp();
	if(nextLevel != 255) {
	  newElem.value = session[i]->getStream(j)->getLevel(nextLevel)->getQuality();
	  newElem.streamIndex = j;
	  newElem.sessionIndex = i;
	  streamSort.push_back(newElem);
	}
      }
    }
  }


  // ===== sort the flows by their Quality values ===================================
  sort(streamSort.begin(), streamSort.end());
  

  // scale up the streams as long as possible
  while(!streamSort.empty()) { 
      
    sessionIndex = streamSort[0].sessionIndex;
    streamIndex    = streamSort[0].streamIndex;

    scaleUpOk = session[sessionIndex]->getStream(streamIndex)->scaleUp();
    
    if(scaleUpOk) {
      scaleCount++;
      //cout << "a" << endl;
      level   = session[sessionIndex]->getStream(streamIndex)->getLevel();
      //cout << "b" << endl;
      nextLevel = session[sessionIndex]->getStream(streamIndex)->getLevel(level)->getLevelUp();
      //cout << "nextLevel " << nextLevel << endl;
      if(nextLevel != 255) {
	quality = session[sessionIndex]->getStream(streamIndex)->getLevel(nextLevel)->getQuality();
	streamSort[0].value = quality;
	sort(streamSort.begin(), streamSort.end());   // <-------------- zu langsam, schneller: durch-bubblen      
      } else {
	streamSort.erase(streamSort.begin());
      }
    } else {
      streamSort.erase(streamSort.begin());   // stream could not longer scaled up   
    }
  }

  //cout << "c" << endl;
  /*
  (*scaleOut) << "new mapping: scaleCount=" << scaleCount << endl;
  (*qualityOut) << "new mapping:" << endl;
  for(i=0; i < (int)session.size(); i++) {                // for each session
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {     // for each flow in this session
      (*qualityOut) << "Stream " << session[i]->getStream(j)->getID()<< " : "<< (int)session[i]->getStream(j)->getQuality();
      (*qualityOut) << "(Level " << (int)session[i]->getStream(j)->getLevel() << ")"<< endl;
      
    }
  }
  */  
  applicationCallback();
  //cout << "2" << endl;

  //reset lossrates of each class in each session
  for(i=0; i < (int)session.size(); i++) {
    for(j=0; j < NUM_PRIOS; j++) {
      session[i]->setLossRateOfClass(j,0);
    }
  }

  // =============================================================================================

}


void QoSManager::addStreamToClasses(LayeredStream* newStreamP, card32 sessionIndex, int level)
{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : addStreamToClasses" << endl;
  #endif
  int i=0,j=0;

  Level* newLevelP = newStreamP -> getLevel(level);


  // cout << "1" << endl;

  // remember old levels
  for(i=0; i < (int) session.size(); i++) {
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {
      session[i]->getStream(j)->rememberOldLevel();
    }
  }

  //==============================

  vector<sortStruct> streamSort;
  sortStruct newElem;

  LayeredStream* streamP;
  Level*         levelP;
  
  int lev;

  int counter=0;
  int remember;

  for(i=0; i < (int)session.size(); i++) {                // for each session
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {     // for each flow in this session
      newElem.sessionIndex = i;
      newElem.streamIndex = j;
      streamP = session[i]->getStream(j);
      lev = streamP -> getLevel();

      if(newStreamP == streamP) {
	remember = counter;
	newElem.value = newStreamP -> getLevel(level) -> getQuality();
	newElem.newStream = true;
	streamSort.push_back(newElem);
      } else {
	if(lev != 255) {
	  newElem.value = streamP -> getLevel(lev) -> getQuality();
	} else {
	  newElem.value = 0;
	}
	newElem.newStream = false;
	streamSort.push_back(newElem);
      }
      counter ++;
    }
  }
  
  card32 maxIndex;
  bool streamOk = false;
  

  while(! newStreamP -> getLevel(level) -> findClasses() ) {
    sort(streamSort.begin(), streamSort.end());
    maxIndex = streamSort.size()-1;
    for(i = maxIndex; i >= 0; i--) {
      if(streamSort[i].newStream == true) {
	level--;
	scaleCount++;
	if(level < 0) break;
	if(level >= 0) {
	  newLevelP = newStreamP -> getLevel(level);
	  streamSort[i].value = newLevelP -> getQuality();
	  break;
	}  
      } else {       
	streamP = session[streamSort[i].sessionIndex]->getStream(streamSort[i].streamIndex);
	lev = streamP -> getLevel();
	levelP = streamP->getLevel(lev);

	// usefull to scale down this stream ?
	streamOk = false;
	for(j=0; j < (int) levelP->getNumLayers() ; j++) {
	  if(newLevelP->classOk( levelP->getLayer(j)->getPriorityClass() )) {
	    streamOk = true;
	    break;
	  }
	}
	if(streamOk) {
	  if( streamP -> scaleDown() ) {
	    scaleCount++;
	    lev = streamP -> getLevel();
	    streamSort[i].value = streamP -> getLevel(lev) -> getQuality();
	    break;
	  }
	}	
      } 
    }
    if(level < 0) {
      newStreamP->setLevel(255);
      level = 255;
      break;
    }
  }

  newStreamP->setLevel(level);

  //cout << "test" << endl;


    
  // try to set streams to their old levels
  for(i=0; i < (int) session.size(); i++) {
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {
      session[i]->getStream(j)->resetToOldLevel();
    }
  }
    
  
  // possible to scale up the new flow?  
  bool scaleUpOk = true;
  while(scaleUpOk) {
    scaleUpOk = newStreamP->scaleUp();
  }
  

}


void QoSManager::setFractionLost(int ID, double fraction, card8 layerNumber)
{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : setFractionLost" << endl;
  #endif
  //cout << "setFractionLost" << endl;
  synchronized();
  int i=0,j=0;

  /*
  static card64 lastCallTime = 0;
  card64 time = getMicroTime();

  if(time - lastCallTime < 500000) {
    unsynchronized();
    return;
  } 

  lastCallTime = time;
  */
  
  //cout << "layer " << (int)layerNumber << " lossrate:" << fraction << endl;

  LayeredStream* streamP;
  card8          level;
  card8          class_;
  
  for(i=0; i < (int) session.size(); i++) {
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {
      streamP = session[i]->getStream(j);
      if(streamP->getID() == (card32)ID) {

	level = streamP->getLevel();

	//cout << "level " << (int) level << " layer " << (int) layerNumber << endl;

	if(level != 255) {
	  if(layerNumber >= streamP->getLevel(level)->getNumLayers()) {
	    unsynchronized();
	    return;
	  }
	  class_ = streamP->getLevel(level)->getLayer(layerNumber)->getPriorityClass();	
	  session[i]->updateLossRateOfClass(class_, (card8) (fraction * 255));
	  if(streamP->getLevel(level)->getLayer(layerNumber)->getLossRate() < fraction*255) {
	    //cout << "scale down !!" << endl;
	    if(streamP->scaleDown()) (*scaleOut) << "packet losses due to scale down of stream " << ID << endl;
	    streamP->setMaximalLevel( streamP->getLevel() );
	    applicationCallback();
	  }
	}
	
	unsynchronized();
	//cout << "setFractionLost beendet" << endl;
	return;
      }
    }
  }
  unsynchronized();
}


int QoSManager::updateStream(const int ID, const ExtendedTransportInfo& ti)
{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : updateStream" << endl;
  #endif
    /*  
  synchronized();

  int i=0,j=0;
  LayeredStream* streamP;
  QoSManagerInterface* interface;
  ExtendedTransportInfo transportinfo;

  //transportinfo = (ExtendedTransportInfo) ti;

  for(i=0; i < (int) session.size(); i++) {
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {
      streamP = session[i]->getStream(j); 
      if(streamP->getID() == (card32) ID) {
	transportinfo = streamP->getTransportInfo();	
	if(!(ti == transportinfo)) {
#ifdef DEBUG_MODE
	  cout << "update" << endl;
#endif
	  interface = streamP->getInterface();
	  //transportinfo = streamP->getTransportInfo();	
	  closeStream(ID);
	  newStream(ID,ti,interface);
	} else {
#ifdef DEBUG_MODE
	  cout << "no update" << endl;
#endif
	}
	unsynchronized();
	return 0;      
      }
    } 
  }

  unsynchronized();
    */
  return 0; 
}


int QoSManager::setWeight(int flowID, short weight, bool newCalc) { 
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : setWeight" << endl;
  #endif

    return 1; 
}



void QoSManager::timerEvent()

{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : timerEvent" << endl;
  #endif

  synchronized();

  // calcualte a new mapping every n seconds 
  // cout << "timer" << endl;
  static int i=0;
  //cout << "qostimer " << i << endl;

  //if(session.size() == 1) 
  //  setFractionLost(1, 0.5, 0);
  //setFractionLost(2, 0.5, 0);
  //setFractionLost(3, 0.5, 0);


  
  if(i >= (int)calculationInterval) {
    i=0;
    //(*outfile) << getMicroTime() / 1000 << " new mapping" << endl;
#ifdef PRINT_INFOS
    cout << "QoSManager: new mapping" << endl;
#endif
    calculateNewMapping();
    //cout << "new mapping done" << endl;

  }
  
  //  writeLogInfo();

  i++;

  //cout << "unsynchronized" << endl;
  unsynchronized();
}


// needed for the sort command of the stl
bool operator< (sortStruct a, sortStruct b) {
  return a.value < b.value;
}


void QoSManager::writeLogInfo() {
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : writeLogInfo" << endl;
  #endif
  
  int i,j;

  int tos;

  //update all RTT's
  for(i=0; i < (int)session.size(); i++) {
    InternetAddress address(session[i]->getDestination());
    for(j=0; j < 1; j++) {
      tos = ResvMod.Get_Priority(j);
      //session[i]->setDelayOfClass(j, rttp->getRoundTripTime(session[i]->getDestination(), tos)/2 );
      (*outfile) << "RTT to " << address << " : " << rttp->getRoundTripTime(session[i]->getDestination(), tos) <<  endl; 
    }
  }
  

  /*
  Flow *flowP;
  Layer *layerP;
  int level;
  int fps;

  (*outfile) << endl;

  for(i=0; i < (int)session.size(); i++) {
    (*outfile) << "session " << session[i].sessionID << " :" << endl;
    for(j=0; j < (int) session[i].flow.size(); j++) {
      flowP = &(session[i].flow[j]);
      level = (int) flowP -> level;
      (*outfile) << "  flow " << flowP -> flowID;
      (*outfile) << ", level " << level << endl;
      fps = (int)(flowP -> ti.start_fps * flowP -> ti.QualityLevel[level].fps_scale);

      for(k=0; k < (int)flowP -> ti.QualityLevel[level].layer.size(); k++) {
	layerP = &(flowP -> ti.QualityLevel[level].layer[k]);
	
	(*outfile) << "    layer " << k << ": size " << 8*layerP->getBandwidth(fps);
	(*outfile) << " bit/sec   class " << (int)layerP->priorityClass << endl ;
      }
    }
  }

  (*outfile) << endl;

  (*outfile) << "free in priority classes: " << endl;
  for(i=0; i < NUM_PRIOS; i++) {
    (*outfile) << "class " << i << " : " << freeInPrio[i]*8 << " bit/sec" << endl;
  }

  (*outfile) << endl;
  */
}


void QoSManager::applicationCallback()
{
  #ifdef DEBUG_MODE
    cout << "Aufruf der Funktion : applicationCallback" << endl;
  #endif


  int i=0,j=0,k=0;
  card8 level;

  int trafficClassOfLayer[4];

  card32 bytes=0;

  LayeredStream* streamP;
  
  ExtendedTransportInfo ti_new;

  card8 layerNumber;

  for(i=0; i < (int) session.size(); i++) {
    for(j=0; j < (int) session[i]->getNumStreams(); j++) {

      streamP = session[i]->getStream(j);
      ti_new = streamP -> getTransportInfo();
      
      level = streamP->getLevel();
      if(level != 255) {
	//bytes = streamP -> getLevel(level)->getLayer(0)->getFrameSize() * streamP -> getLevel(level)->getFPS();

	layerNumber = streamP->getLevel(level)->getNumLayers();


	//cout << "FPS Start " << (int)streamP -> getStartFPS() << endl;;
	//cout << "FrameSize " << streamP -> getLevel(level)->getLayer(0)->getFrameSize() << endl;
	//extensionBytes = flowP -> ti.QualityLevel[level].layer[1].frameSize;

	// basicPrio = streamP ->getLevel(level)->getLayer(0)->getPriorityClass();
	// extensionPrio = flowP->ti.QualityLevel[level].layer[1].priorityClass;
	
	//ti_new = streamP -> getTransportInfo();

	for(k=0; k < (int) ti_new.QualityLevel[0].MaxQualityLayers; k++) {
	  ti_new.WantedBytesPerSecond[k].setValue(0);	  
	}


	for(k=0; k < (int) layerNumber; k++) {
	  
	  bytes = streamP -> getLevel(level)->getLayer(k)->getFrameSize() * streamP -> getLevel(level)->getFPS();
	 
	  //cout << "framesize of layer "<< k << " : " << streamP -> getLevel(level)->getLayer(k)->getFrameSize() << endl;
	  
	  trafficClassOfLayer[k] = streamP ->getLevel(level)->getLayer(k)->getPriorityClass();
	  ti_new.WantedBytesPerSecond[k].setValue(bytes);
	  // cout << "Layer " << k << " : " << endl;
	  // cout << "  Prio : " << trafficClassOfLayer[k] << endl;
	  // cout << "  bytes : " << bytes << endl << endl;
	}

	ResvMod.Get_Priority(&ti_new ,trafficClassOfLayer, layerNumber );

	//cout << "traffic class : " << (int)ti_new.TrafficClass[2] << endl;

	//ti_new.WantedBytesPerSecond[0].setValue(bytes);

	// cout << "bytes :" << bytes << endl;
	//.WantedExtensionBytesPerSecond = extensionBytes;

	
      } else {
	for(k=0; k < (int) getMaxLayerNumber() ; k++) { 
	  ti_new.WantedBytesPerSecond[k].setValue(0);
	//ti_new.WantedExtensionBytesPerSecond = 0;  
	}  
      }
      //cout << endl << "QOS: new flow bw: " << flowP->ti_alt.WantedBytesPerSecond.getValue() << endl;
      streamP->getInterface()->adaptTransportInfo(streamP->getID(), ti_new);
    }
  }


  //for(i=0; i<16; i++) {
  //  cout << "free in class " << i << " : " << freeInPrio[i] << endl;
  //}
}


// ?????????????????????????????????????????
void QoSManager::setReport(int flowID, const RTCPReceptionReportBlock* report, card8 layerNumber)
{
   setFractionLost(flowID,report->getFractionLost(),layerNumber);
}
// ?????????????????????????????????????????


}
