//########################################################
//##                                                    ##
//##                 QoS Management                     ##
//##                 ==============                     ##
//##                                                    ##
//## Session                                            ##
//##                                                    ##
//## Version 1.0                                        ##
//## Date  : March 2000                                 ##
//## Author: Simon Vey (vey@cs.uni-bonn.de)             ##
//##                                                    ##
//########################################################

#ifndef SESSION_H
#define SESSION_H

#include <vector>
#include "layeredstream.h"
#include "../internetaddress.h"
#include "qosglobals.h"

namespace Coral {

class QoSManager;

/**
 * This class manages one session in the QoS-Manager. 
 *
 * @short   Session
 * @author  Simon Vey (vey@cs.uni-bonn.de)
 * @version 1.0
 * @see
 */

class Session {
 public:
  /**
   * Constructor.
   * 
   * @param _qm Pointer to the QoSManager base class (interface to QoSManager).
   */
  Session(QoSManager* qm_, PortableAddress dest);


  /**
   * Creates a new stream in the session
   *
   * @param ti TransportInfo of the flow.
   * @param interface Interface for application-callback
   * @return flow identifier
   */
  card32 newStream(card32 ID, ExtendedTransportInfo ti, QoSManagerInterface* interface);
  

  bool   closeStream(card32 ID);
  /**
   * Deletes a stream from the session.
   *
   * @param id Flow identifier
   * @return true, if flow has been closed, false otherwise
   */
  //bool closeStream(card32 id);

  //=== getNumFlows ==========================================================
  /**
   * Returns the current number of flows in this session.
   *   
   * @return Current number of flows in this session.
   */
  card32 getNumStreams() const;

  void    setID(card32 ID);
  card32  getID();

  void    setWeight(card32 weight_);
  card32  getWeight();

  void    addStream(LayeredStream* str);
  //  bool    removeStream(card32 ID);

  void    setDelayOfClass(card8 class_, card32 delay);
  card32  getDelayOfClass(card8 class_);
  card32  updateDelayOfClass(card8 class_, card32 measuredDelay);

  void    setLossRateOfClass(card8 class_, card8 lossRate);
  card8   getLossRateOfClass(card8 class_);
  card8   updateLossRateOfClass(card8 class_, card8 lossRate);

  LayeredStream* getStream(int index);

  QoSManager* getQoSManager();

  card64  getBandwidth();

  PortableAddress getDestination();

 private:
  QoSManager* qm;

  PortableAddress destination;

  card32 id;

  card32 weight;

  vector<LayeredStream*> stream;

  card32 delayOfClass[NUM_PRIOS];
  card8  lossRateOfClass[NUM_PRIOS];

};

}

#include "session.icc"
#endif
