// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Main Control                                                     ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


import java.io.*;
import java.util.*;
import java.lang.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


import TransportInfo;
import AudioClientMain;
import Prog4DClient;
import BandwidthDivider;
import Information;


/**
  * This class starts a AudioClientGUIs, the Prog4d client and controls
  * the total bandwidth.
  *
  * @short   Main Control
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
public class MainControl extends JFrame
   implements ActionListener, AdjustmentListener
{
   private long Bandwidth = 1000000;

   private JButton buttonQuit            = new JButton("Quit");
   private JButton buttonInformation     = new JButton("Information");
   private JButton buttonHelp            = new JButton("Help");
   private JScrollBar scrollbarBandwidth = new JScrollBar(JScrollBar.HORIZONTAL,(int)(Bandwidth / 1000) ,100,0,3100);
   private JLabel labelBandwidth;

   private final int sourceCount = 5;
   private JButton[] buttonSourceStart    = new JButton[sourceCount];
   private JLabel[]  labelSourceBandwidth = new JLabel[sourceCount];
   private JLabel[]  labelSourceWanted    = new JLabel[sourceCount];
   private JLabel[]  labelSourceMaximum   = new JLabel[sourceCount];
   private JLabel    labelTotalBandwidth  = new JLabel("0",JLabel.RIGHT);
   private JLabel    labelTotalWanted     = new JLabel("0",JLabel.RIGHT);
   private JLabel    labelTotalMaximum    = new JLabel("0",JLabel.RIGHT);

   private Information information           = new Information();
   private Prog4DClient prog4dclient         = null;
   private AudioClientGUI[] audioClient      = new AudioClientGUI[sourceCount - 1];
   private TransportInfo[] transportInfo     = new TransportInfo[sourceCount];
   private long[][] clientBandwidth          = new long[sourceCount][TransportInfoLevel.MaxQualityLayers];
   private BandwidthDivider bandwidthDivider = new BandwidthDivider();

   private String audioLocation;
   private String videoLocation;


   // ###### Constructor ####################################################
   /**
     * Constructor.
     *
     * @param audioURL Default URL for audio streams (e.g. rtpa://gaffel:7500/Test.list) or null.
     * @param videoURL Default URL for video streams or null.
     */
   public MainControl(String audioURL, String videoURL) {
      audioLocation = audioURL;
      videoLocation = videoURL;

      // ====== Button panel ================================================
      getContentPane().setLayout(new GridLayout(5 + sourceCount,1));
      JPanel panelControlButtons = new JPanel();
      panelControlButtons.setLayout(new GridLayout(1,3));
      panelControlButtons.add(buttonQuit);
      panelControlButtons.add(buttonInformation);
      panelControlButtons.add(buttonHelp);
      buttonQuit.addActionListener(this);
      buttonInformation.addActionListener(this);
      buttonHelp.addActionListener(this);
      getContentPane().add(panelControlButtons);

      // ====== Bandwidth panel =============================================
      JPanel panelBandwidth = new JPanel();
      panelBandwidth.setLayout(new GridLayout(1,2));
      JLabel labelBandwidthTitle = new JLabel("Total Bandwidth Limit:");
      labelBandwidth = new JLabel("" + Bandwidth + " [Bytes/s]");
      panelBandwidth.add(labelBandwidthTitle);
      panelBandwidth.add(labelBandwidth);
      getContentPane().add(panelBandwidth);

      scrollbarBandwidth.addAdjustmentListener(this);
      getContentPane().add(scrollbarBandwidth);

      // ====== Source panel =================================================
      JPanel panelSourceHeader = new JPanel();
      panelSourceHeader.setLayout(new GridLayout(1,5));
      panelSourceHeader.add(new JLabel(" Control:"));
      panelSourceHeader.add(new JLabel(" Source:"));
      panelSourceHeader.add(new JLabel("Bandwidth:",JLabel.RIGHT));
      panelSourceHeader.add(new JLabel("Wanted:",JLabel.RIGHT));
      panelSourceHeader.add(new JLabel("Maximum:",JLabel.RIGHT));
      getContentPane().add(panelSourceHeader);

      JPanel panelSource[]       = new JPanel[sourceCount];
      JLabel[]  labelSourceTitle = new JLabel[sourceCount];
      for(int i = 0;i < sourceCount;i++) { 
         transportInfo[i] = new TransportInfo();
         panelSource[i] = new JPanel();
         panelSource[i].setLayout(new GridLayout(1,5));

         buttonSourceStart[i] = new JButton("Start Client");
         buttonSourceStart[i].addActionListener(this);

         if(i > 0) {
            labelSourceTitle[i] = new JLabel(" Audio #" + i);
            audioClient[i - 1] = null;
         }
         else {
            labelSourceTitle[i] = new JLabel(" Prog4D Video");
         }
         labelSourceBandwidth[i] = new JLabel("0",JLabel.RIGHT);
         labelSourceWanted[i]    = new JLabel("0",JLabel.RIGHT);
         labelSourceMaximum[i]   = new JLabel("0",JLabel.RIGHT);

         panelSource[i].add(buttonSourceStart[i]);          
         panelSource[i].add(labelSourceTitle[i]);
         panelSource[i].add(labelSourceBandwidth[i]);
         panelSource[i].add(labelSourceWanted[i]);
         panelSource[i].add(labelSourceMaximum[i]);
         getContentPane().add(panelSource[i]);
      }

      JPanel totalPanel = new JPanel();
      totalPanel.setLayout(new GridLayout(1,5));
      JLabel labelNull  = new JLabel();
      JLabel labelTotal = new JLabel(" Total:");
      totalPanel.add(labelNull);
      totalPanel.add(labelTotal);
      totalPanel.add(labelTotalBandwidth);
      totalPanel.add(labelTotalWanted);
      totalPanel.add(labelTotalMaximum);
      getContentPane().add(totalPanel);

      // ====== Show window =================================================
      setTitle("RTP Audio/Video Bandwidth Control");
      setSize(620,280); 
      show();
   }


   // ###### AdjustmentListener's adjustmentValueChanged() implementation ###
   /**
     * AdjustmentListener's adjustmentValueChanged() implementation.
     */
   synchronized public void adjustmentValueChanged(AdjustmentEvent e) {
      Bandwidth = 1000 * scrollbarBandwidth.getValue();
      labelBandwidth.setText("" + Bandwidth + " [Bytes/s]");
   }


   // ###### ActionListener's actionPerformed() implementation ##############
   /**
     * ActionListener's actionPerformed() implementation.
     */
   synchronized public void actionPerformed(ActionEvent e) {
      String arg = e.getActionCommand();
      
      // ====== Handle start events =========================================
      for(int i = 0;i < sourceCount;i++) {
         if(e.getSource() == (Object)buttonSourceStart[i]) {
            // ====== Start/stop Prog 4D ====================================
            if(i == 0) {
               if(prog4dclient == null) {
                  prog4dclient = new Prog4DClient(videoLocation);
                  prog4dclient.play();
                  buttonSourceStart[i].setLabel("Stop Client");
               }
               else {
                  prog4dclient.stop();
                  prog4dclient = null;
                  buttonSourceStart[i].setLabel("Start Client");
               }
            }
            // ====== Start/stop RTP Audio ==================================
            else {
               if(audioClient[i - 1] != null) {
                  if(!audioClient[i - 1].isClosed()) {
                     audioClient[i - 1].quit();
                     audioClient[i - 1] = null;
                     buttonSourceStart[i].setLabel("Start Client");
                     return;
                  }
                  audioClient[i - 1] = null;
               }
               int device = -1;
               if(i == 1)
                  device = 0;
               audioClient[i - 1] = new AudioClientGUI(audioLocation,device,false);
               audioClient[i - 1].play();
               buttonSourceStart[i].setLabel("Stop Client");
            }
            return;
         }
      }

      // ====== Handle control button events ================================
      if(arg.equals("Quit"))             quit();
      else if(arg.equals("Information")) information();
      else if(arg.equals("Help"))        help();
   }


   // ###### Quit MainControl ###############################################
   /**
     * Quit MainControl.
     */
   synchronized public void quit() {
      if(prog4dclient != null) {
         prog4dclient.stop();
         prog4dclient = null;
      }
      for(int i = 0;i < sourceCount - 1;i++) {
         if(audioClient[i] != null) {
            audioClient[i].quit();
            audioClient[i] = null;
         }
      }
      // Note: Due to Java 1.3 exit() bug, it is necessary to kill java
      //       sending SIGINT to current process.
      AudioClient.killJavaInterpreter();
      // System.exit(0);
   }


   // ###### Information ####################################################
   /**
     * Show information window.
     */
   synchronized public void information() {
      information.show("Information","Copyright","Thomas Dreibholz","Dreibholz@bigfoot.com");
   }


   // ###### Help ###########################################################
   /**
     * Show help window.
     */
   synchronized public void help() {
      information.show("Help",
         "Main Control help:\n\n" +
         "1. Click on \"Start Client\" to start a client.\n" +
         "2. Move bandwidth scrollbar to set total bandwidth\n" +
         "   limit.\n");
   }


   // ###### Divide up bandwidth ############################################
   /**
     * Divide up bandwidth.
     */
   synchronized public void divideUpBandwidth() {
      // ====== Get TransportInfos ==========================================
      if(prog4dclient != null)
         prog4dclient.getTransportInfo(transportInfo[0]);
      else
         transportInfo[0].reset();

      for(int i = 0;i < sourceCount - 1;i++) {
         if((audioClient[i] != null) && (!audioClient[i].isClosed())) {
            audioClient[i].getTransportInfo(transportInfo[i + 1]);
         }
         else {
            audioClient[i] = null;
            transportInfo[i + 1].reset();
         }
      }

      // ====== Calculate and set bandwidth limit for each client ===========
      bandwidthDivider.divideUp(Bandwidth,transportInfo,clientBandwidth,sourceCount);
      long totalValue  = 0;
      long totalWanted = 0;
      long totalMax    = 0;
      for(int i = 0;i < sourceCount;i++) {
         long wanted = transportInfo[i].WantedBytesPerSecond[0];
         long max    = transportInfo[i].MaxBytesPerSecond[0];
         long value  = clientBandwidth[i][0];
         for(int j = 1;j < transportInfo[i].QualityLayers;j++) {
            wanted += transportInfo[i].WantedBytesPerSecond[j];
            max    += transportInfo[i].MaxBytesPerSecond[j];
            value  += clientBandwidth[i][j];
         }
         totalValue  += value;
         totalWanted += wanted;
         totalMax    += max;
         labelSourceBandwidth[i].setText("" + value);
         labelSourceWanted[i].setText("" + wanted);
         labelSourceMaximum[i].setText("" + max);
         if(i == 0) {
            if(prog4dclient != null)
               prog4dclient.setBandwidthLimit(value);
         }
         else {
            if(audioClient[i - 1] != null)
               audioClient[i - 1].setBandwidthLimit((int)value);
         }
      }
      labelTotalBandwidth.setText("" + totalValue);
      labelTotalWanted.setText("" + totalWanted);
      labelTotalMaximum.setText("" + totalMax);
   }


   // ###### Main program ###################################################
   /**
     * Main program.
     */
   public static void main(String args[]) {
      // ====== Get arguments ===============================================
      String audioLocation = null;
      String videoLocation = null;
      if(args.length >= 1) {
         audioLocation = args[0];
      }
      if(args.length >= 2) {
         videoLocation = args[1];
      }

      // ====== Start MainControl ===========================================
      MainControl mc = new MainControl(audioLocation,videoLocation);

      // ====== Main loop ===================================================      
      for(;;) {
         mc.divideUpBandwidth();
         try {
            Thread.sleep(250);
         }
         catch(Exception e) { }
      }
   }
}
   