// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Congestion Manager                                               ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "timedthread.h"
#include "socket.h"
#include "internetaddress.h"
#include "congestionmonitorreport.h"
#include "congestionmanagerpacket.h"
#include "tools.h"
#include "congestionmanager.h"
#include "breakdetector.h"


using namespace Coral;


// ###### Main program ######################################################
int main(int argc, char** argv)
{
   char*  local   = NULL;
   char*  monitor = NULL;
   card16 port    = 7601;

   // ====== Get arguments ==================================================
   for(cardinal i = 1;i < (cardinal)argc;i++) {
      if(!(strncasecmp(argv[i],"-local=",7))) {
          local = &argv[i][7];
      }
      else if(!(strncasecmp(argv[i],"-monitor=",9))) {
          monitor = &argv[i][9];
      }
      else if(!(strncasecmp(argv[i],"-port=",6))) {
          port = atoi(&argv[i][6]);
      }
      else {
         cerr << "Wrong parameter: " << argv[i] << endl;
         cerr << "Usage: " << argv[0] << endl
              << "       [-monitor=host:port} {-port=port} {-local=hostname}"
              << endl;
         exit(1);
      } 
   }
   if(monitor == NULL) {
      cerr << "ERROR: No congestion monitor given (-monitor=host:port)!" << endl;
      exit(1);
   }

   // ====== Creating socket for managers to be managed ======================
   InternetAddress monitorAddress(monitor);
   InternetAddress managerAddress(local,port);
   Socket          managerSocket(Socket::IP,Socket::UDP);
   if(managerSocket.bind(managerAddress) == false) {
      cerr << "ERROR: Unable to bind socket to address "
           << managerAddress << "!" << endl;
      exit(1);
   }

   CongestionManager congestionManager(&monitorAddress,&managerSocket);
   installBreakDetector();
   congestionManager.start();


   // ====== Print status ===================================================
   cout << "Congestion Manager - Copyright (C) 1999-2001 Thomas Dreibholz" << endl;
   cout << "-------------------------------------------------------------" << endl;
   cout << endl;
   cout << "Manager address:  " << managerAddress << endl;
   cout << "Monitor address:  " << monitorAddress << endl;
   cout << endl;


   // ====== Main loop ======================================================
   while(!breakDetected()) {
      Thread::delay(250000,true);
   }


   // ====== Clean up =======================================================
   congestionManager.stop();
   cout << "Terminated!" << endl;
   return(0);
}
