package bonn.infoIV.v3d2.buffer;

import java.util.Arrays;

public class DoubleBufferedObject
{
  public static final int MAX_VERTEX_COUNT        = 7154;
  private static final int MAX_TRIANGLE_COUNT      = 14304;
  private static final int MAX_BASE_VERTEX_COUNT   = 55;
  private static final int MAX_BASE_TRIANGLE_COUNT = 106;
  private static final int MAX_VERTEX_SPLIT_COUNT  = 7124;

  public static final byte FRAME_ONE      = 0;
  public static final byte FRAME_TWO      = 1;
  public static final int  MAX_INFO_SIZE  = 5;

  public static final byte ZERO    =  0;  // this is a 0 for filling the Arrays, maybe obsolete
  public static final int  NO_SIZE = -7;  // If no new Size for the array is wanted, NO_SIZE is given

  protected int[]     ArraySizes_ = null; // The sizes of all n arrays, but you can also code like a.length=b[1]+b[2].
  protected boolean[] Writable_   = null; // A trace of which array is readable and which is not.
  protected int[]     MaxValues_  = null; // an array of the MAX_???_COUNT values, now a FAKE!!!
  public    byte[][]  Arrays_     = null; // You can have up to 63 byte-arrays of arbitary size double-buffered.
  protected int[]     ActualSplitCount_ = null; // an array of the actual count, of splits received

  private int Offset_  = 0;
  private int MaxSize_ = 0;

  private int fuubar=0;


  public DoubleBufferedObject(int sizes, int arrayCount)
  {
    super();
    ArraySizes_ = new int[sizes*2];
    Arrays_     = new byte[arrayCount*2][];
    Writable_   = new boolean[arrayCount];
    MaxValues_  = new int[MAX_INFO_SIZE];
    ActualSplitCount_ = new int[arrayCount];

    // FAKE_ME_BEGIN
    MaxValues_[0] = MAX_VERTEX_COUNT;
    MaxValues_[1] = MAX_TRIANGLE_COUNT;
    MaxValues_[2] = MAX_BASE_VERTEX_COUNT;
    MaxValues_[3] = MAX_BASE_TRIANGLE_COUNT;
    MaxValues_[4] = MAX_VERTEX_SPLIT_COUNT;
    Arrays_[0] = new byte[(MaxValues_[2]+MaxValues_[3])*Object3D.BASE_ENTITY];
    Arrays_[1] = new byte[(MaxValues_[2]+MaxValues_[3])*Object3D.BASE_ENTITY];
    Arrays_[2] = new byte[MaxValues_[4]*Object3D.SPLIT_ENTITY];
    Arrays_[3] = new byte[MaxValues_[4]*Object3D.SPLIT_ENTITY];
    // FAKE_ME_END
    for(int i=0 ; i<Arrays_.length; i++) Arrays.fill(Arrays_[i], ZERO);
    Arrays.fill(Writable_, true);
    Arrays.fill(ActualSplitCount_, ZERO);
  }


  public int[] getMaxInfo()
  {
    return MaxValues_;
  }

  public int getArraySize(byte array)
  {
    return Arrays_[array].length;
  }

  public void setInfo(boolean frame, int[] count)
  {
    if(frame)
    {
      for(int i=0 ; i<count.length ; i++)
      {
        ArraySizes_[i] = count[i];
      }
    }
    else
    {
      for(int i=0 ; i<count.length ; i++)
      {
        ArraySizes_[i+count.length] = count[i];
      }
    }
  }

  public int[] getInfo(boolean frame)
  {
    int[] actual = new int[ArraySizes_.length/2];
    if(frame)
    {
      for(int i=0 ; i<actual.length ; i++)
      {
        actual[i] = ArraySizes_[i];
      }
    }
    else
    {
      for(int i=0 ; i<actual.length ; i++)
      {
        actual[i] = ArraySizes_[i+actual.length];
      }
    }
    //System.out.println("Gespeichert: Vertices: " +actual[0] +" Triangles: " +actual[1] +" Splits: " +actual[2]);
    return actual;
  }


  public int getReceivedSplitCount(boolean frame) 
  { 
    if(frame) return ActualSplitCount_[FRAME_ONE];
    else return ActualSplitCount_[FRAME_TWO];
  }


  public void setArray(byte array, boolean frame, byte[] data, boolean resetOffset, int newSize, int maxInPacket)
  {
    int maxRead = 0; // you can read this far the packet. Either maxInPacket or ArraySizes_[i]. The smaller!

    if(newSize!=NO_SIZE) MaxSize_ = newSize; // MaxSize is the maximum valid size for this array in this Frame.
    if(resetOffset) Offset_ = 0;             // The Offset or Position of the first element has to be reseted
    if(MaxSize_<maxInPacket) maxRead = MaxSize_; // How far can we read the actual data
    else
    {
      maxRead = maxInPacket;
      MaxSize_ -= maxInPacket;
    }

    if(frame)
    {
      if(array==Object3D.VERTEX_SPLITS) ActualSplitCount_[FRAME_ONE] += maxRead; // NICHT SCHOEN!!!
      else ActualSplitCount_[FRAME_ONE] = ZERO;
      for(int j=4 ; j<(maxRead+4) ; j++) // the Server sends an int to specify the Offset ->packet-order issue
      {
        Arrays_[array][Offset_] = data[j];
        Offset_++;
      }
    }
    else // the same in case of the other Buffer
    {
      if(array==Object3D.VERTEX_SPLITS) ActualSplitCount_[FRAME_TWO] += maxRead; // NICHT SCHOEN!!!
      else ActualSplitCount_[FRAME_TWO] = ZERO;
      for(int j=4 ; j<(maxRead+4) ; j++)
      {
        Arrays_[array+1][Offset_] = data[j];
        Offset_++;
      }
    }
  // [PENDING] Hier koenne folgende DEBUG-AUSGABE Routinen folgen, welche evtl Fehlerhaft sind...
  }



//       if(type==Object3D.BASE_MESH_V)
//       {
//        System.out.println("MaxRead: " +maxRead +" ArraySize: " +ArraySizes_[type] +" maxInPaket: " +maxInPacket);
//        float[] v = new float[3];
//        for(int i=0; i<ArraySizes_[type] ; i++) {
//        v[0] = Float.intBitsToFloat(((Arrays_[array][12*i] & 0xff)<<24)
//                                   +((Arrays_[array][12*i+1] & 0xff)<<16)
//                                   +((Arrays_[array][12*i+2] & 0xff)<<8)
//                                   +((Arrays_[array][12*i+3] & 0xff)));
//        v[1] = Float.intBitsToFloat(((Arrays_[array][12*i+4] & 0xff)<<24)
//                                   +((Arrays_[array][12*i+5] & 0xff)<<16)
//                                   +((Arrays_[array][12*i+6] & 0xff)<<8)
//                                   +((Arrays_[array][12*i+7] & 0xff)));
//        v[2] = Float.intBitsToFloat(((Arrays_[array][12*i+8] & 0xff)<<24)
//                                   +((Arrays_[array][12*i+9] & 0xff)<<16)
//                                   +((Arrays_[array][12*i+10] & 0xff)<<8)
//                                   +((Arrays_[array][12*i+11] & 0xff)));
//        System.out.println("Geschrieben: Knoten No.:" +i +" ist:" +v[0] +"," +v[1] +"," +v[2]);
//        }
//       }




//     int z=0;
//     int i=0;
//     int all=0;
//     int splitIndex = 0;
//     float splitCoord = 0f;
//     int complete = 0;


//if(array==Object3D.VERTEX_SPLITS)
//  System.out.println("Der Offset ist gleich " +Offset_ +" beim Array " +array +" im Frame 1.");
//if(array==Object3D.BASE_MESH)
//  System.out.println("Der Offset ist gleich " +Offset_ +" beim Array " +array +" im Frame 1.");
//AUSGABE
//         if((array==Object3D.VERTEX_SPLITS))
// 	{
//           if(!(z<3))
// 	  {
//             if(all<3)
// 	    {
//               splitIndex = ((data[j-3] & 0xff)<<24)
//                           +((data[j-2] & 0xff)<<16)
//                           +((data[j-1] & 0xff)<<8)
//                           +((data[j]   & 0xff));
//               System.out.print(" [" +splitIndex +"] ");
//               all++;
// 	    }
//             else if(all<6)
// 	    {
//               splitCoord = Float.intBitsToFloat(((data[j-3] & 0xff)<<24)
//                                                +((data[j-2] & 0xff)<<16)
//                                                +((data[j-1] & 0xff)<<8)
//                                                +((data[j]   & 0xff)));
//               System.out.print(" [" +splitCoord +"] ");
//               all++;
// 	    }
//             else
// 	    {
//               all=1;
//               System.out.println(" ist VertexSplit " +i);
//               i++;
//               splitIndex = ((data[j-3] & 0xff)<<24)
//                           +((data[j-2] & 0xff)<<16)
//                           +((data[j-1] & 0xff)<<8)
//                           +((data[j]   & 0xff));
//               System.out.print(" [" +splitIndex +"] ");
//             }
//             z=0;
//           }
//           else z++;
// 	}

//         if(array==Object3D.BASE_MESH)
// 	{
//           if(!(z<3))
// 	  {
//             if(type)
// 	    {
//               splitIndex = ((data[j-3] & 0xff)<<24)
//                           +((data[j-2] & 0xff)<<16)
//                           +((data[j-1] & 0xff)<<8)
//                           +((data[j]   & 0xff));
//               System.out.print(" [" +splitIndex +"] ");
//               complete++;
//               if(complete==3)
// 	      {
//                 System.out.println(" Dreieck :" +fuubar);
//                 complete = 0;
//                 fuubar++;
//               }
// 	    }
//             else
// 	    {
//               splitCoord = Float.intBitsToFloat(((data[j-3] & 0xff)<<24)
//                                                +((data[j-2] & 0xff)<<16)
//                                                +((data[j-1] & 0xff)<<8)
//                                                +((data[j]   & 0xff)));
//               System.out.print(" [" +splitCoord +"] ");
//               complete++;
//               if(complete==3)
// 	      {
//                 System.out.println(" Knoten: " +fuubar);
//                 complete = 0;
//                 fuubar++;
//               }
// 	    }
//             z=0;
//           }
//           else z++;
// 	}
//AUSGABE ENDE


//if(array==Object3D.VERTEX_SPLITS)
//  System.out.println("Der Offset ist gleich " +Offset_ +" beim Array " +array +" im Frame 2.");
//if(array==Object3D.BASE_MESH)
//  System.out.println("Der Offset ist gleich " +Offset_ +" beim Array " +array +" im Frame 2.");

//AUSGABE
//         if((array==Object3D.VERTEX_SPLITS))
// 	{
//           if(!(z<3))
// 	  {
//             if(all<3)
// 	    {
//               splitIndex = ((data[j-3] & 0xff)<<24)
//                           +((data[j-2] & 0xff)<<16)
//                           +((data[j-1] & 0xff)<<8)
//                           +((data[j]   & 0xff));
//               System.out.print(" [" +splitIndex +"] ");
//               all++;
// 	    }
//             else if(all<6)
// 	    {
//               splitCoord = Float.intBitsToFloat(((data[j-3] & 0xff)<<24)
//                                                +((data[j-2] & 0xff)<<16)
//                                                +((data[j-1] & 0xff)<<8)
//                                                +((data[j]   & 0xff)));
//               System.out.print(" [" +splitCoord +"] ");
//               all++;
// 	    }
//             else
// 	    {
//               all=1;
//               System.out.println(" ist VertexSplit " +i);
//               i++;
//               splitIndex = ((data[j-3] & 0xff)<<24)
//                           +((data[j-2] & 0xff)<<16)
//                           +((data[j-1] & 0xff)<<8)
//                           +((data[j]   & 0xff));
//               System.out.print(" [" +splitIndex +"] ");
//             }
//             z=0;
//           }
//           else z++;
// 	}

//         if(array==Object3D.BASE_MESH)
// 	{
//           if(!(z<3))
// 	  {
//             if(type)
// 	    {
//               splitIndex = ((data[j-3] & 0xff)<<24)
//                           +((data[j-2] & 0xff)<<16)
//                           +((data[j-1] & 0xff)<<8)
//                           +((data[j]   & 0xff));
//               System.out.print(" [" +splitIndex +"] ");
//               complete++;
//               if(complete==3)
// 	      {
//                 System.out.println(" Dreieck: " +fuubar);
//                 complete = 0; fuubar++;
//               }
// 	    }
//             else
// 	    {
//               splitCoord = Float.intBitsToFloat(((data[j-3] & 0xff)<<24)
//                                                +((data[j-2] & 0xff)<<16)
//                                                +((data[j-1] & 0xff)<<8)
//                                                +((data[j]   & 0xff)));
//               System.out.print(" [" +splitCoord +"] ");
//               complete++;
//               if(complete==3)
// 	      {
//                 System.out.println(" Knoten: " +fuubar);
//                 complete = 0; fuubar++;
//               }
// 	    }
//             z=0;
//           }
//           else z++;
// 	}
//AUSGABE ENDE



  public void resetArray(byte arrayId, int length)
  {
    for ( int i=0 ; i<length ; i++) Arrays_[arrayId][i] = ZERO;
  }


  public void setStatus(boolean array)
  {
    if(array) { Writable_[FRAME_ONE] = !Writable_[FRAME_ONE]; }
    else Writable_[FRAME_TWO] = !Writable_[FRAME_TWO];
  }

  public boolean getStatus(boolean array)
  {
    if(array) { return Writable_[FRAME_ONE]; }
    else return Writable_[FRAME_TWO];
  }
}


