package bonn.infoIV.v3d2.rtp;

import java.io.DataInputStream;
import java.io.DataOutputStream;

/**
 * This interface is the basis of every Channel, be it a DataChannel or a ControlChannel.
 * It supplies a field ChannelType_ for distinguishing implementations of various transport-protocols.<br>  
 * It also defines the method processPacket, whereas it determines the concept of processing the revceived data. <br>
 * The concept of processing is based on InputStreams, particulary a DataInputStream.
 */
public interface Channel
{
  /** The Buffersize of any Socketimplementation. This is multiplied with the maximal packetsize bound to arrive. */
  public static final int BUFFER_SIZE = 50;

  /** BASE_CHANNEL is the base state of ChannelType_. Later, as a specific protocol is implemented this changes. */
  public static final byte BASE_CHANNEL = -1;

  /** Indicating, that the implemented transport protocol is UDP ~ Ethernet. */
  public static final byte UDP_CHANNEL  =  0;

  /** Indicating, that the implemented transport protocol is TCP ~ Ethernet. */
  public static final byte TCP_CHANNEL  =  1;

  /** Indicating, that the implemented transport protocol is AAL5 ~ ATM. */
  public static final byte ATM_CHANNEL  =  2;

  /** This byte states, which implementation is used for the DataChannel or the ControlChannel */
  public static final byte ChannelType_ = BASE_CHANNEL;


  /**
   * This method should be used, after receiving a Packet.
   * The received Packetdata has to be wraped into a DataInputStream
   *
   * @param in - containing the data-field of the transport-protocol Packet.
   * (i.e. including RTPHeader but excluding UDP-Header)
   */
  public void processPacket(DataInputStream in);


  /** [PENDING]: Wozu diese Methode??? */
  public void createPacket(DataOutputStream out);

  public void shutdown();
}
