// RTP_Packet.hh --- RTP_Packet Class Header File

#ifndef RTP_PACKET_H
#define RTP_PACKET_H

#include <stl_config.h>             // bool
#include <RTP/common.hh>            // Typedefs
#include <byte_order/byte_order.hh> // reverse()

class RTP_Packet
{
private:
#ifndef BYTE_ORDER
#error "BYTE_ORDER undefined !"
#elif BYTE_ORDER == BIG_ENDIAN
  u_int16 v:2;		                  // Protocol Version
  u_int16 p:1;		                  // Padding Flag
  u_int16 x:1;		                  // Header Extension Flag
  u_int16 cc:4;		                  // CSRC Count
  u_int16 m:1;		                  // Marker Bit
  u_int16 pt:7;		                  // Payload Type
#elif BYTE_ORDER == LITTLE_ENDIAN
  u_int16 cc:4;		                  // CSRC Count
  u_int16 x:1;		                  // Header Extension Flag
  u_int16 p:1;		                  // Padding Flag
  u_int16 v:2;		                  // Protocol Version
  u_int16 pt:7;		                  // Payload Type
  u_int16 m:1;		                  // Marker Bit
#else
#error "BYTE_ORDER unknown !"
#endif
  u_int16 seq;                            // Sequence Number
  u_int32 ts;		                  // Timestamp
  u_int32 ssrc;		                  // Synchronization Source
//u_int32 csrc[0];                        // Optional Contribution Source List
public:
  RTP_Packet();
  u_int16 getVersion();
  u_int16 getPadding();
  u_int16 getExtension();
  u_int16 getCSRCCount();
  u_int16 getMarker();
  u_int16 getPayloadType();
  u_int16 getSequenceNum();
  u_int32 getTimestamp();
  u_int32 getSSRC();
//u_int32 getCSRCList();
  void setVersion(u_int16);
  void setPadding(u_int16);
  void setExtension(u_int16);
  void setCSRCCount(u_int16);
  void setMarker(u_int16);		
  void setPayloadType(u_int16);
  void setSequenceNum(u_int16);
  void setTimestamp(u_int32);
  void setSSRC(u_int32);
//void setCSRCList(u_int32);
};

class FEC_Packet : public RTP_Packet
{
private:
  u_int16 snBase;
  u_int16 lnRec;
#ifndef BYTE_ORDER
#error "BYTE_ORDER undefined !"
#elif BYTE_ORDER == BIG_ENDIAN
  u_int32 xBit:1;
  u_int32 ptRec:7;
  u_int32 mask:24;
#elif BYTE_ORDER == LITTLE_ENDIAN
  u_int32 ptRec:7;
  u_int32 xBit:1;
  u_int32 mask:24;
#else
#error "BYTE_ORDER unknown !"
#endif
  u_int32 tsRec;
public:
  u_int16 getSNBase();
  u_int16 getLNRecovery();
  u_int32 getExtensionBit();
  u_int32 getPTRecovery();
  u_int32 getMask();
  u_int32 getTSRecovery();
  void setSNBase(u_int16);
  void setLNRecovery(u_int16);
  void setExtensionBit(u_int32);
  void setPTRecovery(u_int32);
  void setMask(u_int32);
  void setTSRecovery(u_int32);
};

#endif
