#include <RTP/SSI.hh>

SSI::SSI()
{
  ssrc = 0;
  probation = RTP_MIN_SEQUENTIAL;
}

void SSI::init(u_int16 seq)
{
  // Meiner Meinung nach muss es hier received = 1 heissen statt received = 0 !!
  // sonst gibt es einen Packetverlust ganz am Anfang !!
  received = 1;  // allgemein muesste es RTP_MIN_SEQUENTIAL sein !!
  received_prior = 0;
  expected_prior = 0;

  cycles = 0;
  max_seq = seq;
  base_seq = seq - 1;

  bad_seq = RTP_SEQ_MOD + 1;
}

void SSI::update(u_int16 seq)
{
  u_int16 udelta = seq - max_seq;

  /*
   * Source is not valid until MIN_SEQUENTIAL packets with
   * sequential sequence numbers have been received.
   */
  if (probation)
    {
      /* packet is in sequence */
      if (seq == max_seq + 1)
	{
	  probation--;
	  max_seq = seq;
	  if (probation == 0)
	    {
	      init(seq);
	      received++;  // Falls received = RTP_MIN_SEQUENTIAL in init() gesetzt wurde, diese Zeile entfernen !
	      return;
	    }
	}
      else
	{
	  max_seq = seq;
	  probation = RTP_MIN_SEQUENTIAL - 1;
	}
      return;
    }
  else if (udelta < RTP_MAX_DROPOUT + 1)  // Hier RTP_MAX_DROPOUT + 1, da udelta i.a 1
    {                                     // (Achtung udelta - 1 geht nicht wg. overflow bei seq = max_seq)
      /* in order, with permissible gap */
      if (seq < max_seq)
	{
	  /*
	   * Sequence number wrapped - count another 64K cycle.
	   */
	  cycles = cycles + RTP_SEQ_MOD;
	}
      max_seq = seq;
    }
  else if (udelta <= RTP_SEQ_MOD - RTP_MAX_MISORDER)
    {
      /* the sequence number made a very large jump */
      if (seq == bad_seq)
	{
	  /*
	   * Two sequential packets -- assume that the other side
	   * restarted without telling us so just re-sync
	   * (i.e., pretend this was the first packet).
	   */
	  init(seq);
	}
      else
	{
	  bad_seq = (seq + 1) & (RTP_SEQ_MOD - 1);
	  return;
	}
    }
  else
    {
      /* duplicate or reordered packet */
    }
  received++;
}

u_int32 SSI::fractionLost()
{
  u_int32 expected = lastSeqNum() - base_seq + 1;

  u_int32 expected_interval = expected - expected_prior;
  u_int32 received_interval = received - received_prior;

  expected_prior = expected;  
  received_prior = received;

  u_int32 lost_interval = expected_interval - received_interval;

  if (expected_interval == 0 || lost_interval <= 0)
    return 0;
  else
    return (lost_interval << 8) / expected_interval;
}

u_int32 SSI::packetsLost()
{
  return (lastSeqNum() - base_seq + 1) - received;
}

u_int32 SSI::interJitter()
{
  return 0;
}

u_int32 SSI::lastSeqNum()
{
  return cycles + max_seq;
}

u_int32 SSI::SSRC() { return ssrc; }
u_int32 SSI::LSR()  { return lsr; }
u_int32 SSI::DLSR() { return 0; }
