#include <time/time.hh>

Time operator+(const Time &x, const Time &y)
{
  Time result;

  result.tv_sec  = x.tv_sec  + y.tv_sec;
  result.tv_usec = x.tv_usec + y.tv_usec;

  long carry = long (result.tv_usec / 1000000);
  if (carry)
    {
      result.tv_sec  += carry;
      result.tv_usec -= carry * 1000000;
    }

  return result;
}

Time operator+(const Time &x, const double time)
{
  Time result, y;

  y.tv_sec  = long (time);
  y.tv_usec = long ((time - y.tv_sec) * 1000000);

  return result = x + y;
}

Time operator+(const double time, const Time &y)
{
  Time result, x;

  x.tv_sec  = long (time);
  x.tv_usec = long ((time - x.tv_sec) * 1000000);

  return result = x + y;
}

Time operator-(const Time &x, Time &y)
{
  Time result;

  /* Perform the carry for the later subtraction by updating y. */
  if (x.tv_usec < y.tv_usec)
    {
      int nsec   = (y.tv_usec - x.tv_usec) / 1000000 + 1;
      y.tv_usec -= 1000000 * nsec;
      y.tv_sec  += nsec;
    }
  if (x.tv_usec - y.tv_usec > 1000000)
    {
      int nsec   = (y.tv_usec - x.tv_usec) / 1000000;
      y.tv_usec += 1000000 * nsec;
      y.tv_sec  -= nsec;
    }

  /* Compute the time remaining to wait.
     tv_usec is certainly positive. */
  result.tv_sec  = x.tv_sec  - y.tv_sec;
  result.tv_usec = x.tv_usec - y.tv_usec;
  
  return result;
}

bool operator<(const Time &x, const Time &y)
{
  if (x.tv_sec < y.tv_sec) return true;
  if (x.tv_sec > y.tv_sec) return false;

  if (x.tv_usec < y.tv_usec)
    return true;
  else
    return false;
}

bool operator<(const Time &x, const double time)
{
  if (x.tv_sec + x.tv_usec / 1000000 < time)
    return true;
  else
    return false;
}

bool operator<(const double time, const Time &y)
{
  if (time < y.tv_sec + y.tv_usec / 1000000)
    return true;
  else
    return false;
}

bool operator<=(const Time &x, const Time &y)
{
  if (x.tv_sec < y.tv_sec) return true;
  if (x.tv_sec > y.tv_sec) return false;

  if (x.tv_usec <= y.tv_usec)
    return true;
  else
    return false;
}

bool operator<=(const Time &x, const double time)
{
  if (x.tv_sec + x.tv_usec / 1000000 <= time)
    return true;
  else
    return false;
}

bool operator<=(const double time, const Time &y)
{
  if (time <= y.tv_sec + y.tv_usec / 1000000)
    return true;
  else
    return false;
}
