// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Source State Info Implementation                                 ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "sourcestateinfo.h"


namespace Coral {


#define RTP_SEQ_MOD        (1 << 16)
#define RTP_MAX_DROPOUT    3000
#define RTP_MAX_MISORDER   100
#define RTP_MIN_SEQUENTIAL 2


// ###### Constructor #######################################################
SourceStateInfo::SourceStateInfo()
   : SeqNumValidator(RTP_MIN_SEQUENTIAL,RTP_MAX_MISORDER,RTP_MAX_DROPOUT,RTP_SEQ_MOD),
     Synchronizable()
{
   reset();
}


// ###### Initialize table ##################################################
void SourceStateInfo::reset()
{
   LSR                = 0;
   LSRUpdateTimeStamp = 0;
   SSRC               = 0;
   SeqNumValidator::reset();
}


// ###### Calculate DLSR ####################################################
card32 SourceStateInfo::calculateDLSR() const
{
   if(LSRUpdateTimeStamp == 0) {
      return(0);
   }
   else {
      const double delay = (double)getMicroTime() - (double)LSRUpdateTimeStamp;
      const card32 dlsr = (card32)(delay * (65536.0 / 1000000.0));
      return(dlsr);
   }
}


}
