// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Stream Report implementation                                     ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "streamreport.h"
#include "tools.h"


namespace Coral {


// ###### Constructor #######################################################
StreamReport::StreamReport()
{
}


// ###### Reset #############################################################
void StreamReport::reset()
{
   Source.reset();
   Destination.reset();
   FlowLabel    = 0;
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      BytesRaw[i]     = 0;
      BytesPayload[i] = 0;
      Packets[i]      = 0;
   }
}


// ###### Translate #########################################################
void StreamReport::translate()
{
   FlowLabel = translate32(FlowLabel);
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      BytesRaw[i]     = translate64(BytesRaw[i]);
      BytesPayload[i] = translate64(BytesPayload[i]);
      Packets[i]      = translate32(Packets[i]);
   }
}


// ###### == operator #######################################################
int StreamReport::operator==(const StreamReport& report) const
{
   return((Source == report.Source) && (Destination == report.Destination) &&
          (FlowLabel == report.FlowLabel));
}


// ###### != operator #######################################################
int StreamReport::operator!=(const StreamReport& report) const
{
   return((Source == report.Source) && (Destination == report.Destination) &&
          (FlowLabel == report.FlowLabel));
}


// ###### Output operator ###################################################
ostream& operator<<(ostream& os, const StreamReport& report)
{
   os << InternetAddress(report.Source) << " -> " << InternetAddress(report.Destination);
   if(report.FlowLabel != 0) {
      char str[64];
      snprintf((char*)&str,sizeof(str),"  Flowlabel $%05x",report.FlowLabel);
      os << str;
   }
   os << endl;
   os << "   Raw     = ";
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      if(report.BytesRaw[i] != 0) {
         os << report.BytesRaw[i] << "(" << TrafficClassValues::getNameForIndex(i) << ") ";
      }
   }
   os << endl;
   os << "   Payload = ";
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      if(report.BytesPayload[i] != 0) {
         os << report.BytesPayload[i] << "(" << TrafficClassValues::getNameForIndex(i) << ") ";
      }
   }
   os << endl;
   os << "   Packets = ";
   for(cardinal i = 0;i < TrafficClassValues::MaxValues;i++) {
      if(report.Packets[i] != 0) {
         os << report.Packets[i] << "(" << TrafficClassValues::getNameForIndex(i) << ") ";
      }
   }
   os << endl;
   return(os);
}


}
